from ftw.contentpage.testing import FTW_CONTENTPAGE_INTEGRATION_TESTING
from unittest2 import TestCase
from zope.component import queryAdapter
from collective.geo.settings.interfaces import IGeoCustomFeatureStyle


class TestAddressGeoConfigHandler(TestCase):

    layer = FTW_CONTENTPAGE_INTEGRATION_TESTING

    def setUp(self):
        super(TestAddressGeoConfigHandler, self).setUp()
        self.portal = self.layer['portal']
        self.portal_url = self.portal.portal_url()

        self.contentpage = self.portal.get(
            self.portal.invokeFactory('ContentPage', 'contentpage'))
        # Fire all necessary events
        self.contentpage.processForm()

    def _create_addressblock(self):
        addressblock = self.contentpage.get(
            self.contentpage.invokeFactory('AddressBlock', 'addressblock'))
        # Fire all necessary events
        addressblock.processForm()
        addressblock.reindexObject()
        return addressblock

    def test_geo_init_handler(self):
        addressblock = self._create_addressblock()
        adapter = queryAdapter(addressblock, IGeoCustomFeatureStyle)
        self.assertTrue(adapter.use_custom_styles)
        self.assertEquals(adapter.map_viewlet_position, 'fake-manager')

    def tearDown(self):
        super(TestAddressGeoConfigHandler, self).tearDown()
        portal = self.layer['portal']
        portal.manage_delObjects(['contentpage'])



from ftw.contentpage.browser.eventlisting import format_date
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_NAME, TEST_USER_PASSWORD
from plone.testing.z2 import Browser
import transaction
import unittest2 as unittest


class TestEvent(unittest.TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.eventfolder = self.portal.get(self.portal.invokeFactory(
            'EventFolder', 'eventfolder'))
        self.eventfolder.processForm()
        transaction.commit()
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

    def test_creation(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
                TEST_USER_NAME, TEST_USER_PASSWORD, ))
        self.browser.open("%s/createObject?type_name=EventPage" %
            self.eventfolder.absolute_url())
        self.browser.getControl(name='title').value = 'A Title'
        self.browser.getControl(name='endDate_minute').value = ['20']
        self.browser.getControl(name='endDate_hour').value = ['08']
        self.browser.getControl(name='endDate_day').value = ['20']
        self.browser.getControl(name='endDate_month').value = ['05']
        self.browser.getControl(name='endDate_year').value = ['2013']
        self.browser.getControl(name='startDate_minute').value = ['20']
        self.browser.getControl(name='startDate_hour').value = ['07']
        self.browser.getControl(name='startDate_day').value = ['20']
        self.browser.getControl(name='startDate_month').value = ['05']
        self.browser.getControl(name='startDate_year').value = ['2013']
        self.browser.getControl(name='location').value = 'D\xc3\xbcbendorf'
        self.browser.getControl(name='form.button.save').click()
        self.assertEqual(self.browser.url,
            'http://nohost/plone/eventfolder/a-title/')
        event = self.eventfolder.get('a-title')
        self.assertEqual(event.location.encode('utf-8'), 'D\xc3\xbcbendorf')
        self.assertEqual(
            format_date(event.start(), event.end(), event.getWholeDay()),
            '20.05.2013 07:20 - 08:20')



from ftw.testing import MockTestCase
from ftw.contentpage.testing import ZCML_LAYER
from ftw.contentpage.interfaces import IAddressBlock
from ftw.geo.interfaces import IGeocodableLocation
from zope.component import getGlobalSiteManager
from ftw.contentpage.geo import AddressBlockLocationAdapter


class TestGeoAdapter(MockTestCase):

    layer = ZCML_LAYER

    def setUp(self):
        super(TestGeoAdapter, self).setUp()

        self.context = self.providing_stub(IAddressBlock)

        gsm = getGlobalSiteManager()
        gsm.registerAdapter(AddressBlockLocationAdapter)

    def test_no_address_data(self):
        self.expect(self.context.getAddress()).result('')
        self.expect(self.context.getZip()).result('')
        self.expect(self.context.getCity()).result('')
        self.expect(self.context.getCountry()).result('')
        self.replay()

        adapter = IGeocodableLocation(self.context)
        self.assertEquals(adapter.getLocationString(), '')

    def test_address_data(self):
        self.expect(self.context.getAddress()).result('street')
        self.expect(self.context.getZip()).result('zip')
        self.expect(self.context.getCity()).result('city')
        self.expect(self.context.getCountry()).result('country')

        self.replay()

        adapter = IGeocodableLocation(self.context)

        self.assertEquals(adapter.getLocationString(),
                          'street, zip, city, country')



from ftw.upgrade import UpgradeStep


class InstallBrowserlayer(UpgradeStep):

    def __call__(self):
        self.setup_install_profile(
            'profile-ftw.contentpage.upgrades:1405', steps=['browserlayer', ])



from ftw.contentpage import config
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_NAME, TEST_USER_PASSWORD
from plone.testing.z2 import Browser
from Products.CMFCore.utils import getToolByName
from simplelayout.base.views import SimpleLayoutView
from unittest2 import TestCase
from zope.viewlet.interfaces import IViewletManager
from zope.component import queryMultiAdapter
import transaction


class TestContentListing(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestContentListing, self).setUp()
        self.portal = self.layer['portal']
        self.portal_url = self.portal.portal_url()

        self.contentpage = self.portal.get(
            self.portal.invokeFactory('ContentPage', 'contentpage'))
        # Fire all necessary events
        self.contentpage.processForm()
        transaction.commit()

        # Browser setup
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

    def _auth(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD, ))

    def _get_viewlet(self):
        view = SimpleLayoutView(self.contentpage, self.contentpage.REQUEST)
        manager_name = 'simplelayout.base.listing'
        manager = queryMultiAdapter(
            (self.contentpage, self.contentpage.REQUEST, view),
            IViewletManager,
            manager_name)
        self.failUnless(manager)
        # Set up viewlets
        manager.update()
        name = 'ftw.contentpage.contentlisting'
        return [v for v in manager.viewlets if v.__name__ == name]

    def test_component_registered(self):
        self.assertTrue(len(self._get_viewlet()) == 1)

    def test_category_field(self):
        self.assertIn('content_categories', self.contentpage.Schema())
        self.contentpage.Schema()['content_categories'].set(
            self.contentpage, 'DEMO1')
        self.assertEquals(
            self.contentpage.Schema()['content_categories'].get(
                self.contentpage),
            ('DEMO1',))

    def test_category_index(self):
        catalog = getToolByName(self.portal, 'portal_catalog')
        for name, meta_type in config.INDEXES:
            self.assertIn(name, catalog.indexes())

        subpage = self.contentpage.get(
            self.contentpage.invokeFactory('ContentPage', 'subpage'))
        subpage.processForm()
        subpage.setTitle('Subpage title')
        subpage.Schema()['content_categories'].set(subpage, 'DEMO1')
        subpage.reindexObject()
        self.assertTrue(catalog({'content_categories': 'DEMO1'})[0])

    def test_category_index_umlauts(self):
        catalog = getToolByName(self.portal, 'portal_catalog')
        subpage = self.contentpage.get(
            self.contentpage.invokeFactory('ContentPage', 'subpage'))
        subpage.processForm()
        subpage.setTitle('Subpage title')
        subpage.Schema()['content_categories'].set(
            subpage, 'WITH UTF8 \xc3\xa4')
        subpage.reindexObject()
        unique_values = catalog.Indexes['getContentCategories'].uniqueValues()
        self.assertIn("WITH UTF8 \xc3\xa4", unique_values)

    def test_viewlet(self):
        viewlet = self._get_viewlet()[0]
        # Should be empty
        self.assertFalse(viewlet.available())

        # Create more content
        subpage = self.contentpage.get(
            self.contentpage.invokeFactory(
                'ContentPage', 'subpage', description='SubpageDescription'))
        subpage.setTitle('Subpage')
        # Test with umlauts
        subpage.Schema()['content_categories'].set(
            subpage, '\xc3\xa4u\xc3\xa4')
        subpage.reindexObject()

        subpage2 = self.contentpage.get(
            self.contentpage.invokeFactory('ContentPage', 'subpage2'))
        subpage2.setTitle('Subpage2')
        subpage2.Schema()['content_categories'].set(subpage2, 'DEMO2')
        subpage2.reindexObject()

        subpage3 = self.contentpage.get(
            self.contentpage.invokeFactory(
                'ContentPage', 'subpage3', description='OtherDescription'))
        subpage3.setTitle('Subpage3')
        subpage3.Schema()['content_categories'].set(subpage3, 'DEMO2')
        subpage3.reindexObject()

        viewlet = self._get_viewlet()[0]
        self.assertTrue(viewlet.available())

        self.assertIn(
            ('\xc3\xa4u\xc3\xa4',
                [('Subpage', subpage.absolute_url(), 'SubpageDescription')]),
            viewlet.get_content())

        self.assertIn(
            ('DEMO2',
                [('Subpage2', subpage2.absolute_url(), ''),
                 ('Subpage3', subpage3.absolute_url(), 'OtherDescription')]),
            viewlet.get_content())

        transaction.commit()

        self._auth()
        self.browser.open(self.contentpage.absolute_url())
        self.assertIn('\xc3\xa4u\xc3\xa4', self.browser.contents)
        self.assertIn('DEMO2', self.browser.contents)
        self.assertIn('subelements-listing-element', self.browser.contents)

    def test_content_categories_sort_order(self):
        viewlet = self._get_viewlet()[0]

        self.contentpage.invokeFactory(
            'ContentPage', 'subpage1', content_categories="aa")
        self.contentpage.invokeFactory(
            'ContentPage', 'subpage2', content_categories="ab")
        self.contentpage.invokeFactory(
            'ContentPage', 'subpage3', content_categories="zz")
        self.contentpage.invokeFactory(
            'ContentPage', 'subpage4', content_categories="AC")

        expected_order = ['aa', 'ab', 'AC', 'zz']
        contents = self.contentpage.objectValues()
        order = [item[0] for item in viewlet._create_resultmap(contents)]

        self.assertEquals(expected_order, order)

    def test_sort_order(self):

        self.contentpage.get(
            self.contentpage.invokeFactory('ContentPage', 'page1', title="AB",
                content_categories="CONTENT")).processForm()
        self.contentpage.get(
            self.contentpage.invokeFactory('ContentPage', 'page2', title="ZZ",
                content_categories="CONTENT")).processForm()
        self.contentpage.get(
            self.contentpage.invokeFactory('ContentPage', 'page3', title="aa",
                content_categories="CONTENT")).processForm()

        viewlet = self._get_viewlet()[0]
        self.assertEquals([item[0] for item in viewlet.get_content()[0][1]],
            ['aa', 'AB', 'ZZ'])

    def tearDown(self):
        super(TestContentListing, self).tearDown()
        portal = self.layer['portal']
        portal.manage_delObjects(['contentpage'])
        transaction.commit()



from Products.CMFCore.permissions import ManagePortal


DEFAULT_TO_HIDE = [
    'subject', 'relatedItems', 'location', 'language',  # Categorization
    'effectiveDate', 'expirationDate',  # Dates
    'creators', 'contributors', 'rights',  # Contributors
    'allowDiscussion', 'excludeFromNav',  # Settings
    'nextPreviousEnabled',
    ]


def finalize(schema, show=None, hide=None):
    to_hide = DEFAULT_TO_HIDE[:]
    if hide:
        to_hide += hide

    if show:
        for name in show:
            if name in to_hide:
                to_hide.remove(name)

    for name in to_hide:
        if name in schema:
            field = schema[name]
            schema.changeSchemataForField(name, 'default')
            field.widget.visible = {'view': 'invisible', 'edit': 'invisible'}
            field.write_permission = ManagePortal

    # Hide from navigation by default
    schema['excludeFromNav'].default = True



from ftw.contentpage.testing import FTW_CONTENTPAGE_INTEGRATION_TESTING
from plone.registry.interfaces import IRegistry
from simplelayout.base.interfaces import ISimpleLayoutBlock
from unittest2 import TestCase
from zope.component import getUtility
from zope.component import queryMultiAdapter
from plone.registry import Record, field


class TestListingBlockCreation(TestCase):

    layer = FTW_CONTENTPAGE_INTEGRATION_TESTING

    def setUp(self):
        super(TestListingBlockCreation, self).setUp()
        self.portal = self.layer['portal']
        self.portal_url = self.portal.portal_url()

        self.contentpage = self.portal.get(
            self.portal.invokeFactory('ContentPage', 'contentpage'))
        # Fire all necessary events
        self.contentpage.processForm()

    def _create_listingblock(self):
        listingblock = self.contentpage.get(
            self.contentpage.invokeFactory('ListingBlock', 'listingblock'))
        # Fire all necessary events
        listingblock.processForm()

        return listingblock

    def test_fti(self):
        self.assertIn('ListingBlock', self.portal.portal_types.objectIds())

    def test_creation(self):
        _id = self.contentpage.invokeFactory('ListingBlock', 'listingblock')
        self.assertIn(_id, self.contentpage.objectIds())

    def test_simplelayout_integration(self):
        listingblock = self._create_listingblock()
        ISimpleLayoutBlock.providedBy(listingblock)

    def test_exclude_from_nav(self):
        listingblock = self._create_listingblock()
        self.assertTrue(listingblock.getExcludeFromNav())

    def test_get_columns(self):
        listingblock = self._create_listingblock()
        self.assertEquals(
            ['getContentType', 'Title', 'modified', 'Creator',
             'getObjSize', 'review_state', 'id'],
            listingblock.getColumns().keys())
        self.assertEquals(
            ['column_type', 'column_title', 'column_modified',
             'column_creater', 'column_size', 'review_state', 'ID'],
            listingblock.getColumns().values())

    def test_default_title(self):
        listingblock = self._create_listingblock()
        # Default is empty
        self.assertEquals('', listingblock.Title())

    def test_can_set_default_page(self):
        listingblock = self._create_listingblock()
        self.assertFalse(listingblock.canSetDefaultPage())

    def test_change_default_title(self):
        registry = getUtility(IRegistry)
        registry.records['ftw.contentpage.listingblock.defaulttitle'] = \
            Record(field.TextLine(title=u"dummy", default=u"N/A"),
                   value=u'Downloads')

        listingblock = self._create_listingblock()
        self.assertEquals('Downloads', listingblock.Title())

    def test_default_table_columns(self):
        registry = getUtility(IRegistry)

        registry.records[
            'ftw.contentpage.listingblock.defaulttablecolumns'].value = \
            (u'Creator', u'Title', u'BadColumn')

        listingblock = self._create_listingblock()

        view = queryMultiAdapter((listingblock, listingblock.REQUEST),
                                 name='block_view')

        content = view.render_table()

        self.assertIn(u'Creator', content)
        self.assertIn(u'Title', content)
        self.assertNotIn(u'BadColumn', content)

    def test_sort_index_vocabulary(self):
        listingblock = self._create_listingblock()

        self.assertEquals(listingblock.getSortIndexVocabulary().keys(),
            ['getContentType', 'sortable_title', 'modified',
             'id', 'getObjPositionInParent'])

    def tearDown(self):
        super(TestListingBlockCreation, self).tearDown()
        portal = self.layer['portal']
        portal.manage_delObjects(['contentpage'])



from DateTime import DateTime
from ftw.builder import Builder
from ftw.builder import create
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import plone
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.testing.z2 import Browser
from Products.CMFCore.utils import getToolByName
from unittest2 import TestCase
import transaction


class TestNewsListing(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.portal.invokeFactory('NewsFolder', 'nf1')
        self.context = self.portal.nf1

        # Create user with local Contributor role
        mtool = getToolByName(self.portal, 'portal_membership', None)
        mtool.addMember('john', 'password', ['Member'], [])
        self.portal.manage_permission('Access inactive portal content',
                                      ['Contributor', 'Manager'],
                                      acquire=False)
        self.context.manage_setLocalRoles('john', ['Contributor'])

        # Create user without special roles
        mtool.addMember('jack', 'password', ['Member'], [])

        self.context.invokeFactory('News', 'n1')
        self.context.invokeFactory('News', 'n2', effectiveDate=DateTime()+1)
        self.context.n2.reindexObject()
        transaction.commit()
        self._set_allowAnonymousViewAbout_property(True)

    def login(self):
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (TEST_USER_NAME, TEST_USER_PASSWORD,))

    def _set_allowAnonymousViewAbout_property(self, value):
        site_props = getToolByName(self.context, 'portal_properties').site_properties
        site_props._updateProperty('allowAnonymousViewAbout', value)
        transaction.commit()

    def is_author_visible(self):
        self.browser.open(self.context.absolute_url())
        return '<span class="documentAuthor"' in self.browser.contents

    def test_logged_in_user_sees_author_when_allowAnonymousViewAbout(self):
        self.login()
        self.assertTrue(self.is_author_visible(),
                        'Logged in user should see author if allowAnonymousViewAbout is True.')

    def test_logged_in_user_sees_author_when_not_allowAnonymousViewAbout(self):
        self.login()
        self._set_allowAnonymousViewAbout_property(False)
        self.assertTrue(self.is_author_visible(),
                        'Logged in user should see author if allowAnonymousViewAbout is False.')

    def test_anonymous_user_sees_author_when_allowAnonymousViewAbout(self):
        self.assertTrue(self.is_author_visible(),
                        'Anonymous user should see author if allowAnonymousViewAbout is True.')

    def test_anonymous_user_dont_sees_author_when_not_allowAnonymousViewAbout(self):
        self._set_allowAnonymousViewAbout_property(False)
        self.assertFalse(self.is_author_visible(),
                         'Anonymous user should not see author if allowAnonymousViewAbout is False.')

    def test_inactive_news_is_visible_for_contributor(self):
        self.browser.addHeader('Authorization', 'Basic john:password')
        self.browser.open(self.context.absolute_url())
        self.assertIn('n2', self.browser.contents)

    def test_inactive_news_is_not_visible_for_regular_users(self):
        self.browser.addHeader('Authorization', 'Basic jack:password')
        self.browser.open(self.context.absolute_url())
        self.assertNotIn('n2', self.browser.contents)

    @browsing
    def test_title_of_news_listing(self, browser):
        page = create(Builder('content page').titled('Contentpage'))
        newsfolder = create(Builder('news folder').within(page).titled('News'))

        browser.login().visit(page, view='@@news_listing')
        self.assertEquals('Contentpage - News', plone.first_heading())

        browser.visit(newsfolder)
        self.assertEquals('News', plone.first_heading())



from DateTime import DateTime
from ftw.contentpage.browser.baselisting import extend_query_by_date
from ftw.contentpage.testing import FTW_CONTENTPAGE_INTEGRATION_TESTING
from plone.app.testing import TEST_USER_ID
import datetime
import os
import unittest2 as unittest


class TestExtendQueryByDate(unittest.TestCase):

    def test_extend_query_by_date(self):
        query = extend_query_by_date({}, '2012/12/01', 'effective')
        self.assertEquals(
            {'effective':
                {'query': (
                    DateTime('2012/12/01').earliestTime(),
                    DateTime('2012/12/31').latestTime()),
                    'range': 'minmax'}},
            query)

    def test_query_does_not_lose_informations_if_date_is_incorrect(self):
        query = extend_query_by_date(
            {'Title': 'James'}, 'not a date', 'effective')
        self.assertEquals(query, {'Title': 'James'})


class TestNewsViews(unittest.TestCase):

    layer = FTW_CONTENTPAGE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        self.newsfolder = self.portal.get(self.portal.invokeFactory(
                'NewsFolder', 'newsfolder'))
        image_ = open("%s/dummy.png" % os.path.split(__file__)[0])

        self.news = self.newsfolder.get(self.newsfolder.invokeFactory(
                'News', 'news', title="My News", image=image_))
        self.news1 = self.newsfolder.get(self.newsfolder.invokeFactory(
                'News', 'news1', title="My News1", image=image_,
                effectiveDate=(
                    datetime.datetime.now() - datetime.timedelta(7))))
        self.news2 = self.newsfolder.get(self.newsfolder.invokeFactory(
                'News', 'news2', title="My News2", image=image_,
                effectiveDate=(
                    datetime.datetime.now() - datetime.timedelta(14))))
        self.news3 = self.newsfolder.get(self.newsfolder.invokeFactory(
                'News', 'news3', title="My News3", image=image_,
                effectiveDate=(
                    datetime.datetime.now() - datetime.timedelta(21))))
        self.news4 = self.newsfolder.get(self.newsfolder.invokeFactory(
                'News', 'news4', title="News without effective date",
                effectiveDate=None))

    def test_news_listing_view_no_restriction(self):
        listing = self.newsfolder.restrictedTraverse("@@news_listing")
        self.assertEqual(len(listing.get_items()), 5)
        brain = listing.get_items()[0]
        self.assertEqual(brain.Title, 'My News')
        self.assertTrue(listing.has_img(brain))
        self.assertIn(
            '<img src="http://nohost/plone/newsfolder/news/@@images/',
            listing.get_img(brain))
        self.assertEqual('test_user_1_', listing.get_creator(brain))

    def test_get_creator_no_member(self):
        listing = self.newsfolder.restrictedTraverse("@@news_listing")
        self.news.setCreators(['dummy'])
        self.news.reindexObject()

        brain = listing.get_items()[0]
        self.assertEquals('dummy', listing.get_creator(brain))

    def test_get_creator_has_fullname(self):
        listing = self.newsfolder.restrictedTraverse("@@news_listing")
        member = self.portal.portal_membership.getMemberById(TEST_USER_ID)
        member.setProperties(fullname='Firstname Lastname')

        brain = listing.get_items()[0]
        self.assertEquals('Firstname Lastname', listing.get_creator(brain))

    def test_news_listing_on_portal(self):
        listing = self.portal.restrictedTraverse("@@news_listing")
        self.assertEqual(len(listing.get_items()), 5)



from ftw.contentpage.interfaces import IEventListingView
from ftw.contentpage.portlets.base_archive_portlet import ArchiveSummary
from plone.app.portlets.portlets import base
from plone.memoize.instance import memoize
from plone.portlets.interfaces import IPortletDataProvider
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.interface import implements


class IEventArchive(IPortletDataProvider):
    """Archive portlet interface.
"""


class Assignment(base.Assignment):
    implements(IEventArchive)

    @property
    def title(self):
        return "Event Archive Portlet"


class Renderer(base.Renderer):

    def __init__(self, context, request, view, manager, data):
        self.context = context
        self.data = data
        self.request = request
        self.view = view

    @property
    def available(self):
        """Only show the portlet:
        - If there are Events
        - If view is EventListing
        """
        has_events = bool(self.archive_summary())

        if IEventListingView.providedBy(self.view):
            return has_events
        else:
            return False

    @memoize
    def archive_summary(self):
        return ArchiveSummary(self.context,
                               self.request,
                               ['ftw.contentpage.interfaces.IEventPage'],
                               'start',
                               'event_listing'
                               )()

    render = ViewPageTemplateFile('event_archive.pt')


class AddForm(base.NullAddForm):

    def create(self):
        return Assignment()



from Products.CMFCore.utils import getToolByName
from collective.geo.settings.interfaces import IGeoFeatureStyle
from collective.geo.settings.interfaces import IGeoSettings
from ftw.contentpage.config import INDEXES
from plone.app.portlets.storage import PortletAssignmentMapping
from plone.portlets.constants import CONTENT_TYPE_CATEGORY
from plone.portlets.interfaces import IPortletManager
from plone.registry.interfaces import IRegistry
from simplelayout.portlet.dropzone.portlets import drop_zone_portlet
from zope.component import getUtility
import logging

LOGGER = logging.getLogger('ftw.contentpage')


def installed(site):
    add_catalog_indexes(site)
    add_georef_settings(site)
    set_dropzone_as_type_portlet(site)
    set_calendar_types(site)


def uninstalled(site):
    remove_catalog_indexes(site)
    remove_georef_settings(site)
    remove_dropzone_as_type_portlet(site)
    remove_calendar_types(site)


def add_catalog_indexes(context):
    """Method to add our wanted indexes to the portal_catalog.

    @parameters:

    When called from the import_various method below, 'context' is
    the plone site and 'logger' is the portal_setup logger.  But
    this method can also be used as upgrade step, in which case
    'context' will be portal_setup and 'logger' will be None.
    """

    catalog = getToolByName(context, 'portal_catalog')
    indexes = catalog.indexes()
    # Specify the indexes you want, with ('index_name', 'index_type')
    indexables = []
    for name, meta_type in INDEXES:
        if name not in indexes:
            catalog.addIndex(name, meta_type)
            indexables.append(name)
            LOGGER.info("Added %s for field %s.", meta_type, name)
    if len(indexables) > 0:
        LOGGER.info("Indexing new indexes %s.", ', '.join(indexables))
        catalog.manage_reindexIndex(ids=indexables)


def remove_catalog_indexes(context):
    catalog = getToolByName(context, 'portal_catalog')
    indexes = catalog.indexes()

    for name, meta_type in INDEXES:
        if name in indexes:
            catalog.delIndex(name)


def add_georef_settings(context):
    """Import step to set up AddressBlock as georeferenceable type in
    colllective.geo.settings.

    This just adds a registry entry, but it can't be done through registry.xml
    because at that point the AddressBlock type hasn't been registered yet.
    """

    registry = getUtility(IRegistry)
    registry.forInterface(
        IGeoFeatureStyle).map_viewlet_position = 'fake-manager'

    settings = registry.forInterface(IGeoSettings)
    if not 'AddressBlock' in settings.geo_content_types:
        types = list(settings.geo_content_types)
        types.append('AddressBlock')
        settings.geo_content_types = types


def remove_georef_settings(context):
    registry = getUtility(IRegistry)

    # only reset map_viewlet_position if fake-manager is still configured
    style = registry.forInterface(IGeoFeatureStyle)
    if style.map_viewlet_position == 'fake-manager':
        geo_default = IGeoFeatureStyle['map_viewlet_position'].default
        style.map_viewlet_position = geo_default

    settings = registry.forInterface(IGeoSettings)
    if 'AddressBlock' in settings.geo_content_types:
        types = list(settings.geo_content_types)
        types.remove('AddressBlock')
        settings.geo_content_types = types


def set_dropzone_as_type_portlet(self):
    manager = getUtility(IPortletManager, name=u"plone.rightcolumn")
    cat = manager[CONTENT_TYPE_CATEGORY]
    if 'ContentPage' not in cat:
        cat['ContentPage'] = PortletAssignmentMapping()
    mapping = cat['ContentPage']
    if 'simplelayout-dropzone-portlet' not in mapping:
        mapping['simplelayout-dropzone-portlet'] = \
            drop_zone_portlet.Assignment()


def remove_dropzone_as_type_portlet(self):
    manager = getUtility(IPortletManager, name=u"plone.rightcolumn")
    cat = manager[CONTENT_TYPE_CATEGORY]
    if 'ContentPage' in cat:
        del cat['ContentPage']


def set_calendar_types(context):
    portal_calendar = getToolByName(context, 'portal_calendar')
    types = list(portal_calendar.calendar_types)
    types.append('EventPage')
    portal_calendar.calendar_types = tuple(types)


def remove_calendar_types(context):
    portal_calendar = getToolByName(context, 'portal_calendar')
    types = list(portal_calendar.calendar_types)
    if 'EventPage' in types:
        types.remove('EventPage')
    portal_calendar.calendar_types = tuple(types)



from DateTime import DateTime
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.contentpage.browser.eventlisting import EventListing
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from ftw.contentpage.testing import FTW_CONTENTPAGE_INTEGRATION_TESTING
from ftw.testing import MockTestCase
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.testing.z2 import Browser
from Products.Five.browser import BrowserView
from pyquery import PyQuery
from unittest2 import TestCase
from zope.component import queryMultiAdapter
from zope.viewlet.interfaces import IViewletManager
import os
import transaction


class TestEventListingIntegration(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.folder = create(Builder('event folder'))

    def test_pending_events_are_still_visible(self):
        event = create(Builder('event page').within(self.folder).having(
            startDate=DateTime()-1,
            endDate=DateTime()+1)
        )

        view = EventListing(self.folder, self.request)

        self.assertEqual(
            [event],
            [brain.getObject() for brain in view.get_items()])

    def test_month_year_filter_on_eventlisting(self):
        event = create(Builder('event page').within(self.folder).having(
            startDate=DateTime('2014/02/02'),
            endDate=DateTime('2014/02/06'))
        )
        view = EventListing(self.folder, self.request)

        self.assertEquals(0, len(view.get_items()))
        self.request.set('archive', '2014/02/01')

        self.assertEquals(1, len(view.get_items()))


class TestEventListing(MockTestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestEventListing, self).setUp()
        self.mock_current_time(2013, 05, 20, 10, 00)

        from DateTime import DateTime

        self.portal = self.layer['portal']
        self.eventfolder = self.portal.get(
        self.portal.invokeFactory('EventFolder', 'eventfolder'))
        self.eventfolder.invokeFactory('EventPage', 'event1', title="Event1",
                                       startDate=DateTime(
                                           2013, 03, 20, 10, 00),
                                       endDate=DateTime(2013, 03, 20, 11, 00),
                                       )
        self.eventfolder.invokeFactory('EventPage', 'event2', title="Event2",
                                       startDate=DateTime(
                                           2013, 05, 22, 10, 00),
                                       endDate=DateTime(2013, 05, 22, 11, 00),
                                       location="Bern")
        file_ = open("%s/dummy.png" % os.path.split(__file__)[0], 'r')
        self.eventfolder.invokeFactory('EventPage', 'event3', title="Event3",
                                       startDate=DateTime(
                                           2013, 05, 24, 10, 00),
                                       endDate=DateTime(2013, 05, 24, 11, 00),
                                       location="Berns",
                                       image=file_)

        self.eventfolder.invokeFactory('EventPage', 'event4', title="Event4",
                                       startDate=DateTime(
                                           2013, 05, 26, 10, 00),
                                       endDate=DateTime(2013, 05, 26, 11, 00),
                                       location="Bern")

        self.eventfolder.invokeFactory('EventPage', 'event5', title="Event5",
                                       startDate=DateTime(
                                           2013, 06, 28, 10, 00),
                                       endDate=DateTime(2013, 06, 28, 11, 00),
                                       location="Bern")

        self.eventfolder.invokeFactory('EventPage', 'event6', title="Event6",
                                       startDate=DateTime(
                                           2013, 05, 30, 10, 00),
                                       endDate=DateTime(2013, 05, 30, 11, 00),
                                       location="Bern")

        transaction.commit()
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

    def mock_current_time(self, *dtargs):
        # since mocker is not able to replace (a.k.a monkeypatch)
        # datetime.datetime.now directly, we need to make a proxy
        # of the datetime class and mix it in by replacing the
        # datetime module...
        from datetime import datetime, timedelta
        pydt_now = datetime(*dtargs)
        pydt_class = self.mocker.proxy(datetime)
        self.expect(pydt_class.now()).result(pydt_now).count(0, None)
        pydt_mod = self.mocker.replace('datetime')
        self.expect(pydt_mod.datetime).result(pydt_class).count(0, None)
        self.expect(pydt_mod.timedelta).result(timedelta).count(0, None)

        from DateTime import DateTime
        zdt_now = DateTime(pydt_now)
        zdt_mod = self.mocker.replace('DateTime')
        self.expect(zdt_mod.DateTime()).result(zdt_now).count(0, None)

        self.mocker.replay()

    def test_listing_no_past_events(self):
        view = self.eventfolder.restrictedTraverse('event_listing')
        events = view.get_items()
        self.assertEqual(len(events), 5)

    def test_listing_sort(self):
        view = self.eventfolder.restrictedTraverse('event_listing')
        events = view.get_items()
        for index, event in enumerate(events):
            if not index == len(events) - 1:
                self.assertTrue(event.start < events[index + 1].start)

    def test_listing_archive(self):
        view = self.eventfolder.restrictedTraverse('event_listing')
        self.portal.REQUEST['archive'] = '2013/06/01'
        events = view.get_items()
        self.assertEqual(len(events), 1)

        from DateTime import DateTime
        self.assertEqual(events[0].start, DateTime(2013, 06, 28, 10, 00))

    def test_has_img(self):
        view = self.eventfolder.restrictedTraverse('event_listing')
        events = view.get_items()
        self.assertFalse(view.has_img(events[0]))
        self.assertTrue(view.has_img(events[1]))

    def test_browser(self):
        self.browser.open(self.eventfolder.absolute_url())
        query = PyQuery(self.browser.contents)
        element = query('.tileImage')
        self.assertEqual(element.attr('alt'), 'Event3')
        self.assertEqual(element.attr('title'), 'Event3')
        self.assertEqual(element.attr('width'), '100')

    @browsing
    def test_whole_day_event(self, browser):
        self.eventfolder.invokeFactory('EventPage', 'event7', title="Event7",
                                       wholeDay=True,
                                       startDate=DateTime(2013, 05, 20, 4, 0),
                                       endDate=DateTime(2013, 05, 20, 5, 0),
                                       location="Bern")
        transaction.commit()
        browser.visit(self.eventfolder)
        event_heading_links = browser.css('.tileHeadline').css('.url')
        event_titles = [link.text for link in event_heading_links]
        self.assertIn('Event7', event_titles)

    def test_event_view(self):
        event1 = self.eventfolder.get('event1')
        event2 = self.eventfolder.get('event2')
        self.browser.open(event1.absolute_url())
        query = PyQuery(self.browser.contents)
        elements = query('.eventdata tr th')
        self.assertEqual(len(elements), 2)
        self.assertEqual(elements[0].text.strip(), 'Date')
        self.assertEqual(elements[1].text.strip(), 'Export')

        self.browser.open(event2.absolute_url())
        query = PyQuery(self.browser.contents)
        elements = query('.eventdata tr th')
        self.assertEqual(len(elements), 3)
        self.assertEqual(elements[0].text.strip(), 'Date')
        self.assertEqual(elements[1].text.strip(), 'Location')
        self.assertEqual(elements[2].text.strip(), 'Export')
        location = query('.eventdata tr td')[-2]
        self.assertEqual(location.text, event2.getLocation())

    def test_event_data_viewlet(self):
        event = self.eventfolder.get('event3')  # has an image
        view = BrowserView(event, event.REQUEST)
        manager_name = 'plone.abovecontentbody'
        manager = queryMultiAdapter((event, event.REQUEST, view),
            IViewletManager,
            manager_name)
        self.failUnless(manager)
        # Set up viewlets
        manager.update()
        name = 'ftw.contentpage.event.eventdata'
        viewlet = [v for v in manager.viewlets if v.__name__ == name][0]

        self.failUnless(viewlet)

    def test_event_archive_portlet(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD, ))

        self.browser.open(
            '%s/++contextportlets++plone.leftcolumn/+/eventarchive' %
            self.eventfolder.absolute_url())

        self.browser.open(self.eventfolder.absolute_url())

        pq = PyQuery(self.browser.contents)
        self.assertTrue(pq('.portlet.portletArchiveListing'),
            'We added one, so there sould be a EventArchive portlet')

        self.assertEquals(
            len(pq(
                '.portlet.portletArchiveListing .portletItem li.month')), 3,
                'Expect three entries in the events archive portlet')



from plone.app.layout.viewlets import content
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from zope.component import getMultiAdapter


class NewsByline(content.DocumentBylineViewlet):

    template = ViewPageTemplateFile('byline.pt')

    def render(self):
        return self.template()

    def creator(self):
        userid = self.context.Creator()
        mt = getToolByName(self.context, 'portal_membership')
        member = mt.getMemberById(userid)
        if member:
            return member.getProperty('fullname') or userid
        return userid

    def getWorkflowState(self):
        state = self.context_state.workflow_state()
        plone_tools = getMultiAdapter(
            (self.context, self.request),
            name='plone_tools')

        workflows = plone_tools.workflow().getWorkflowsFor(self.context)
        if workflows:
            for w in workflows:
                if state in w.states:
                    return w.states[state].title or state



from DateTime import DateTime
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from unittest2 import TestCase

from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING


class TestNewsPortletListing(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.portal.REQUEST
        self.portal_url = self.portal.portal_url()

    def test_no_portlet_found(self):
        self.request.form.update({'portlet': 'not_existing'})
        view = self.portal.unrestrictedTraverse('news_portlet_listing')
        self.assertEqual(None, view.get_portlet())

    def test_portlet_found_in_leftcolumn(self):
        portlet = create(Builder('news portlet'))
        self.request.form.update({'portlet': portlet.__name__,
                                  'manager': u'plone.leftcolumn'})
        view = self.portal.unrestrictedTraverse('news_portlet_listing')
        self.assertEqual(portlet,
                         view.get_portlet().data)

    @browsing
    def test_leftcolumn_portlet(self, browser):
        folder = create(Builder('news folder').titled('News'))
        create(Builder('news').titled('Bookings open').within(folder))

        create(Builder('news portlet')
               .having(only_context=False,
                       path=['/{0}'.format(folder.getId())],
                       more_news_link=True))

        browser.login().open()
        browser.find('More News').click()
        self.assertEquals(['Bookings open'],
                          browser.css('h2.tileHeadline').text)

    @browsing
    def test_rightcolumn_portlet(self, browser):
        folder = create(Builder('news folder').titled('News'))
        create(Builder('news').titled('Bookings open').within(folder))

        create(Builder('news portlet')
               .in_manager(u'plone.rightcolumn')
               .having(only_context=False,
                       path=['/{0}'.format(folder.getId())],
                       more_news_link=True))

        browser.login().open()
        browser.find('More News').click()
        self.assertEquals(['Bookings open'],
                          browser.css('h2.tileHeadline').text)

    @browsing
    def test_newslisting_of_inherited_news_portlet(self, browser):
        # Create a news portlet on root which will be inherited by the root's
        # child objects.
        create(Builder('news portlet')
               .in_manager(u'plone.rightcolumn')
               .within(self.portal)
               .having(portlet_title='Aktuell',
                       more_news_link=True,
                       quantity=5,
                       days=10))

        # Create a content page having a news folder. Create an old news
        # entry in the news folder.
        content_page = create(Builder('content page')
                              .titled('My Content Page'))
        news_folder = create(Builder('news folder')
                             .titled('News')
                             .within(content_page))
        create(Builder('news')
               .titled('Bookings open')
               .having(effectiveDate=DateTime(1999, 11, 15))
               .within(news_folder))

        # Visit the content page containing the news portlet. The old news
        # entry will not be rendered in the portlet due to the config of the
        # news portlet.
        browser.login().visit(content_page)

        # Clicking on the 'More News' link should render a page containing
        # the news entry.
        browser.find('More News').click()
        self.assertEquals(['Bookings open'],
                          browser.css('h2.tileHeadline').text)

    @browsing
    def test_newslisting_title(self, browser):
        """
        This test makes sure that the news listing view renders the
        title of the portlet, not the generic title.
        """
        folder = create(Builder('news folder').titled('News'))
        create(Builder('news').titled('Bookings open').within(folder))

        create(Builder('news portlet')
               .in_manager(u'plone.rightcolumn')
               .having(portlet_title='Breaking News',
                       only_context=False,
                       path=['/{0}'.format(folder.getId())],
                       more_news_link=True))

        browser.login().open()
        browser.find('More News').click()
        self.assertEquals(
            ['Breaking News'],
            browser.css('h1.documentFirstHeading').text
        )

    @browsing
    def test_newslisting_no_description(self, browser):
        """
        This test makes sure that the news listing view does not
        render the generic description.
        """
        content_page = create(Builder('content page')
                              .titled('My Page')
                              .having(description='My Description'))

        folder = create(Builder('news folder').titled('News'))
        create(Builder('news').titled('Bookings open').within(folder))

        create(Builder('news portlet')
               .within(content_page)
               .in_manager(u'plone.rightcolumn')
               .having(only_context=False,
                       path=['/{0}'.format(folder.getId())],
                       more_news_link=True))

        browser.login().visit(content_page)
        browser.find('More News').click()
        self.assertEquals(
            0,
            len(browser.css('div.documentDescription'))
        )

    @browsing
    def test_all_news(self, browser):
        """
        This test makes sure that the number of news entries is not limited
        when rendering "all news".
        """

        # Create a news portlet which limits the number of news entries to one.
        create(Builder('news portlet')
               .in_manager(u'plone.rightcolumn')
               .within(self.portal)
               .having(portlet_title='Aktuell',
                       more_news_link=True,
                       quantity=1))

        # Create a content page having a news folder.
        content_page = create(Builder('content page')
                              .titled('My Content Page'))
        news_folder = create(Builder('news folder')
                             .titled('News')
                             .within(content_page))

        # Create two news entries.
        create(Builder('news').titled('News item 1').within(news_folder))
        create(Builder('news').titled('News item 2').within(news_folder))

        # Visit the content page containing the news portlet.
        browser.login().visit(content_page)

        # Clicking on the 'More News' link should render a page containing
        # both news entries.
        browser.find('More News').click()
        self.assertEquals(
            2,
            len(browser.css('div.newsListing div.tileItem'))
        )

    @browsing
    def test_news_listing_fixes_context_on_news_item(self, browser):
        """
        When news listing is called on a news item it needs to change its
        context for its search starting point. Else it would only display
        the current item.
        """
        create(Builder('news portlet')
               .in_manager(u'plone.rightcolumn')
               .within(self.portal)
               .having(portlet_title='Aktuell',
                       more_news_link=True,
                       quantity=0))

        # Create a content page having a news folder.
        content_page = create(Builder('content page')
                              .titled('My Content Page'))
        news_folder = create(Builder('news folder')
                             .titled('News')
                             .within(content_page))
        item1 = create(
            Builder('news').titled('News item 1').within(news_folder))
        create(Builder('news').titled('News item 2').within(news_folder))
        create(Builder('news').titled('News item 3').within(news_folder))

        browser.open(item1)
        # check items in news portlet
        self.assertEqual(
            ['News item 1', 'News item 2', 'News item 3'],
            browser.css('.portletWrapper .newsTemplate .portletItem .portletItemTitle').text
        )

        browser.find('More News').click()
        # check news items in news listing view
        self.assertEquals(
            ['News item 1', 'News item 2', 'News item 3'],
            browser.css('.newsListing .tileHeadline a').text
        )

    @browsing
    def test_news_listing_renders_expired_items(self, browser):
        news_folder = create(Builder('news folder'))

        # Create six news items.
        create(Builder('news')
               .titled('Default News')
               .within(news_folder))
        create(Builder('news')
               .titled('Default News With Expiration Date')
               .within(news_folder)
               .having(expirationDate=DateTime() + 20))
        create(Builder('news')
               .titled('Future News')
               .within(news_folder)
               .having(effectiveDate=DateTime() + 10))
        create(Builder('news')
               .titled('Future News With Expiration Date')
               .within(news_folder)
               .having(effectiveDate=DateTime() + 10)
               .having(expirationDate=DateTime() + 20))
        create(Builder('news')
               .titled('Old News')
               .within(news_folder)
               .having(effectiveDate=DateTime() - 10))
        create(Builder('news')
               .titled('Old News (Expired)')
               .within(news_folder)
               .having(effectiveDate=DateTime() - 20)
               .having(expirationDate=DateTime() - 10))

        create(Builder('news portlet')
               .in_manager(u'plone.rightcolumn')
               .within(self.portal)
               .having(portlet_title='My Portlet',
                       more_news_link=True,
                       quantity=100))

        # Make sure the admin sees all news entries, even expired and future
        # news items.
        browser.login().open().find('More News').click()
        self.assertEqual(
            [
                'Future News',
                'Future News With Expiration Date',
                'Default News',
                'Default News With Expiration Date',
                'Old News',
                'Old News (Expired)',
            ],
            browser.css('.newsListing .tileHeadline a').text
        )

        # Make sure the anonymous user only sees news entries which are
        # not expired and not in the future.
        browser.logout().open().find('More News').click()
        self.assertEquals(
            [
                'Default News With Expiration Date',
                'Default News',
                'Old News',
            ],
            browser.css('.newsListing .tileHeadline a').text
        )

        # Configure the portlet to show expired news too.
        browser.login()
        browser.visit(self.portal, view='manage-portlets')
        browser.find('News Portlet (My Portlet)').click()
        browser.fill({u'Show expired items': True}).submit()
        browser.forms['form'].fill({u'Show expired items': True}).find('Save').click()

        # Make sure the admin still sees all news entries, even expired
        # and future news items.
        browser.login().open().find('More News').click()
        self.assertEqual(
            [
                'Future News',
                'Future News With Expiration Date',
                'Default News',
                'Default News With Expiration Date',
                'Old News',
                'Old News (Expired)',
            ],
            browser.css('.newsListing .tileHeadline a').text
        )

        # Make sure the anonymous user sees all news entries, even the
        # expired ones. But not the future news.
        browser.logout().open().find('More News').click()
        self.assertEqual(
            [
                'Default News With Expiration Date',
                'Default News',
                'Old News',
                'Old News (Expired)',
            ],
            browser.css('.newsListing .tileHeadline a').text
        )



from AccessControl import ClassSecurityInfo
from ftw.calendarwidget.browser.widgets import FtwCalendarWidget
from ftw.contentpage import _
from ftw.contentpage.config import PROJECTNAME
from ftw.contentpage.content.contentpage import ContentPage
from ftw.contentpage.content.contentpage import ContentPageSchema
from ftw.contentpage.content.textblock import image_schema
from ftw.contentpage.interfaces import IEventPage
from Products.Archetypes import atapi
from Products.ATContentTypes.config import HAS_LINGUA_PLONE
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from zope.interface import implements
from Products.ATContentTypes.lib.calendarsupport import CalendarSupportMixin


if HAS_LINGUA_PLONE:
    from Products.LinguaPlone.public import registerType
else:
    from Products.Archetypes.atapi import registerType


EventSchema = ContentPageSchema.copy() + atapi.Schema((
    atapi.DateTimeField(
        'startDate',
        required=True,
        searchable=False,
        accessor='start',
        languageIndependent=True,
        widget=FtwCalendarWidget(
            helper_js=('++resource++ftw.contentpage.resources/start_end_date_helper.js',),
            label=_(u'label_event_start', default=u'Event Starts'),
            description=_(u'help_start',
                          default=u"Date and Time, when the event begins."),
        ),
    ),

    atapi.DateTimeField(
        'endDate',
        required=True,
        searchable=False,
        accessor='end',
        languageIndependent=True,
        widget=FtwCalendarWidget(
            label=_(u'label_event_end', default=u'Event Ends'),
            description=_(u'help_end',
                          default=u"Date and Time, when the event ends."),
        ),
    ),

    atapi.BooleanField(
        'wholeDay',
        default=False,
        languageIndependent=True,
        widget=atapi.BooleanWidget(
            label=_(u'label_whole_day_event', u'Whole day event'),
            description=_(u'help_whole_day', default=u"Event lasts whole day"),
        ),
    ),
    atapi.StringField(
        'location',
        searchable=True,
        widget=atapi.StringWidget(
            label=_(u'label_event_location', default=u'Event Location'),
            description=_(u'help_event_location', default=u""),
        ),
    ),
))

finalizeATCTSchema(EventSchema)
# finalizeATCTSchema moves 'location' into 'categories', we move it back:
EventSchema.changeSchemataForField('location', 'default')

# Protect the teaser image with a specific permission
permission = "ftw.contentpage: Edit teaser image on EventPage"
for name in image_schema.keys():
    EventSchema[name].write_permission = permission


class EventPage(ContentPage, CalendarSupportMixin):
    implements(IEventPage)

    meta_type = "EventPage"
    schema = EventSchema
    security = ClassSecurityInfo()

    security.declarePublic('show_description')
    def show_description(self):
        return False

    security.declarePrivate('get_addressblock')
    def get_addressblock(self):
        blocks = self.getFolderContents(
            contentFilter={'portal_type': ['AddressBlock']}, full_objects=True)
        if not len(blocks) > 0:
            return None
        return blocks[0]

    security.declareProtected("View", 'contact_name')
    def contact_name(self):
        block = self.get_addressblock()
        if block:
            return block.getAddressTitle()
        return ''

    security.declareProtected("View", 'contact_phone')
    def contact_phone(self):
        block = self.get_addressblock()
        if block:
            return block.getPhone()
        return ''

    security.declareProtected("View", 'contact_email')
    def contact_email(self):
        block = self.get_addressblock()
        if block:
            return block.getEmail()
        return ''

    security.declareProtected("View", 'getLocation')
    def getLocation(self):
        field_value = self.getField('location').get(self)
        if not field_value:
            block = self.get_addressblock()
            complete_address = ''
            if block:
                street = block.getAddress()
                if street:
                    complete_address = complete_address + street + ','
                zip = block.getZip()
                if zip:
                    complete_address = complete_address + ' ' + zip
                city = block.getCity()
                if city:
                    complete_address = complete_address + ' ' + city
                return complete_address.strip(',')
            return ''
        else:
            return field_value

    security.declareProtected("View", 'event_url')
    def event_url(self):
        block = self.get_addressblock()
        if block:
            return block.getWww()
        return ''

registerType(EventPage, PROJECTNAME)



from Acquisition import aq_parent, aq_inner
from DateTime import DateTime
from ftw.contentpage import _
from ftw.contentpage.interfaces import INews
from plone import api
from plone.app.portlets.browser.interfaces import IPortletAddForm
from plone.app.portlets.browser.interfaces import IPortletEditForm
from plone.app.portlets.interfaces import IPortletPermissionChecker
from plone.app.portlets.portlets import base
from plone.formwidget.contenttree import MultiContentTreeFieldWidget
from plone.formwidget.contenttree import PathSourceBinder
from plone.portlets.interfaces import IPortletDataProvider
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import form, button, field, interfaces
from zope import schema
from zope.component import getMultiAdapter
from zope.interface import implements, invariant, Invalid


class INewsPortlet(IPortletDataProvider):

    portlet_title = schema.TextLine(
        title=_(u'Title'),
        description=u'',
        required=True,
        default=u'')

    show_image = schema.Bool(title=_(u'label_show_image'),
                             required=True,
                             default=True)

    path = schema.List(
        title=_(u"Path"),
        description=u"",
        value_type=schema.Choice(
            source=PathSourceBinder(
                navigation_tree_query={
                    'is_folderish': True},
                is_folderish=True),
        ),
        required=False,
    )

    only_context = schema.Bool(title=_(u'label_only_context'),
                               description=_('help_only_context'),
                               default=True,
                               )

    classification_items = schema.List(
        title=_(u"Classification Items"),
        description=u"",
        value_type=schema.Choice(
            source=PathSourceBinder(
                navigation_tree_query={
                    'portal_type': 'ClassificationItem'},
                portal_type='ClassificationItem'),
        ),
        required=False,
    )

    quantity = schema.Int(title=_(u'label_quantity'),
                          default=5)

    subjects = schema.List(
        title=_(u'label_subjects'),
        value_type=schema.Choice(
            vocabulary='ftw.contentpage.subjects',

        ),
        required=False
    )

    show_desc = schema.Bool(title=_(u'label_show_desc',
                                    default=u"Show Description"),
                            default=True)

    desc_length = schema.Int(title=_(u'label_desc_length'),
                             default=50)

    days = schema.Int(title=_(u'label_days', default="Days"),
                      description=_(u'description_days',
                                    default="Show news of the las x days."),
                      default=0,
                      required=True)

    more_news_link = schema.Bool(title=_(u'label_more_news_link',
                                    default=u"Show more news link"),
                            default=False)

    rss_link = schema.Bool(title=_(u'label_rss_link',
                                    default=u"Shows link to the rss feed"),
                            default=False)

    always_render_portlet = schema.Bool(
        title=_(u'label_always_render_portlet'),
        description=_('help_always_render_portlet'),
        default=False,
    )

    show_expired_news = schema.Bool(
        title=_(
            u'label_show_expired_news',
            default=u"Show expired items"
        ),
        description=_(
            'help_show_expired_news',
            default=u"The news portlet and the news listing will also "
                    u"render expired items."
        ),
        default=False,
    )

    @invariant
    def is_either_path_or_area(obj):
        """Checks if not both path and current area are defined.
        """
        if obj.only_context and obj.path:
            raise Invalid(
                _(u'text_path_and_area',
                  default=u'You can not set a path and limit to context.'))


class AddForm(form.AddForm):
    implements(IPortletAddForm)
    label = _(u'Add News Portlet')
    description = _(u'This Portlet displays News')

    fields = field.Fields(INewsPortlet)

    def __init__(self, context, request):
        super(AddForm, self).__init__(context, request)
        self.status = None
        self._finishedAdd = None

    def __call__(self):
        IPortletPermissionChecker(aq_parent(aq_inner(self.context)))()
        return super(AddForm, self).__call__()

    def nextURL(self):
        editview = aq_parent(aq_inner(self.context))
        context = aq_parent(aq_inner(editview))
        url = str(getMultiAdapter((context, self.request),
                                  name=u"absolute_url"))
        return url + '/@@manage-portlets'

    @button.buttonAndHandler(_(u"label_save", default=u"Save"), name='add')
    def handleAdd(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        obj = self.createAndAdd(data)
        if obj is not None:
            # mark only as finished if we get the new object
            self._finishedAdd = True

    @button.buttonAndHandler(_(u"label_cancel", default=u"Cancel"),
                             name='cancel_add')
    def handleCancel(self, action):
        nextURL = self.nextURL()
        return self.request.response.redirect(nextURL)

    def add(self, object_):
        ob = self.context.add(object_)
        self._finishedAdd = True
        return ob

    def updateWidgets(self):
        self.fields['classification_items'].widgetFactory = \
            MultiContentTreeFieldWidget
        self.fields['path'].widgetFactory = MultiContentTreeFieldWidget
        if not self.context.portal_types.get('ClassificationItem', None):
            self.fields['classification_items'].mode = interfaces.HIDDEN_MODE
        else:
            self.fields['classification_items'].mode = interfaces.INPUT_MODE
        super(AddForm, self).updateWidgets()

    def create(self, data):
        return Assignment(
            portlet_title=data.get('portlet_title', 'News'),
            show_image=data.get('show_image', True),
            only_context=data.get('only_context', True),
            quantity=data.get('quantity', 5),
            classification_items=data.get('classification_items', []),
            path=data.get('path', []),
            subjects=data.get('subjects', []),
            show_desc=data.get('show_desc', False),
            desc_length=data.get('desc_length', 50),
            days=data.get('days', 0),
            more_news_link=data.get('more_news_link', 0),
            rss_link=data.get('rss_link', 0),
            always_render_portlet=data.get('always_render_portlet', False,),
            show_expired_news=data.get('show_expired_news', False,),
        )


class Assignment(base.Assignment):
    implements(INewsPortlet)

    def __init__(self, portlet_title="News", show_image=True,
                 only_context=True, quantity=5, classification_items=None,
                 path=None, subjects=None, show_desc=False, desc_length=50,
                 days=0, more_news_link=0, rss_link=0,
                 always_render_portlet=False, show_expired_news=False):
        self.portlet_title = portlet_title
        self.show_image = show_image
        self.only_context = only_context
        self.quantity = quantity
        self.classification_items = classification_items or []
        self.path = path or []
        self.subjects = subjects or []
        self.show_desc = show_desc
        self.desc_length = desc_length
        self.days = days
        self.more_news_link = more_news_link
        self.rss_link = rss_link
        self.always_render_portlet = always_render_portlet
        self.show_expired_news = show_expired_news

    @property
    def title(self):
        """This property is used to display the title of the portlet in the
        "manage portlets" screen. The user defined title of the portlet
        instance is appended to the default title which is useful if there
        is more than one news portlet.
        """
        return u'News Portlet ({0})'.format(self.portlet_title)


class Renderer(base.Renderer):
    render = ViewPageTemplateFile('news_portlet.pt')

    def tag_image(self, brain):
        if not self.data.show_image:
            return ''
        obj = brain.getObject()
        scale = getMultiAdapter((obj, self.request), name=u"images")
        scaled_img = scale.scale('image', scale='thumb', direction='down')

        if scaled_img:
            return scaled_img.tag()
        return ''

    @property
    def available(self):
        if getattr(self.data, 'always_render_portlet', False):
            return True

        is_news = self.context.portal_type == 'NewsFolder'
        if self.show_more_news_link():
            has_news = self.get_news(all_news=True)
        else:
            has_news = self.get_news()
        return has_news and not is_news

    def get_news(self, all_news=False, listing_view=False):
        catalog = getToolByName(self.context, 'portal_catalog')
        url_tool = getToolByName(self.context, 'portal_url')
        portal_path = url_tool.getPortalPath()
        query = {'object_provides': 'ftw.contentpage.interfaces.INews'}

        if self.data.only_context:
            path = '/'.join(self.get_news_context().getPhysicalPath())
            query['path'] = {'query': path}

        else:
            if self.data.path:
                cat_path = []
                for item in self.data.path:
                    cat_path.append('/'.join([portal_path, item]))
                query['path'] = {'query': cat_path}

        if self.data.classification_items:
            cs_uids = []
            for item in self.data.classification_items:
                obj = self.get_news_context().restrictedTraverse(
                    '/'.join([portal_path, item.strip('/')]))
                cs_uids.append(obj.UID())
            query['cs_uids'] = cs_uids

        if self.data.subjects:
            query['Subject'] = self.data.subjects

        if self.data.days > 0 and not all_news:
            date = DateTime() - self.data.days
            query['effective'] = {'query': date, 'range': 'min'}

        show_all = False
        show_inactive = False
        if listing_view and getattr(self.data, 'show_expired_news', False):
            # Prepare to force the query to show inactive content too.
            # See https://www.fourdigits.nl/blog/listing-expired-plone-content
            # for more information.
            show_all = True
            show_inactive = True

            # See https://www.fourdigits.nl/blog/listing-expired-plone-content
            # for the reason behind the following query extension.
            query['expires'] = {
                'query': (DateTime('2000/01/01'), DateTime('2500/01/01')),
                'range': 'min:max',
            }

            # Since we're forcing the query to show future content too, we
            # need to restrict this manually.
            if not api.user.has_permission('Access future portal content'):
                query['effective'] = {'query': DateTime(), 'range': 'max', }

        query['sort_on'] = 'effective'
        query['sort_order'] = 'descending'
        results = catalog.searchResults(
            query,
            show_all=show_all,
            show_inactive=show_inactive,
        )

        if not all_news and self.data.quantity:
            results = results[:self.data.quantity]

        return results

    def crop_desc(self, description):
        ploneview = self.context.restrictedTraverse('@@plone')
        return ploneview.cropText(description, self.data.desc_length)

    def show_more_news_link(self):
        return self.data.more_news_link

    def show_rss_link(self):
        return getattr(self.data, 'rss_link', False)

    def more_news_url(self):
        params = 'portlet={0}&manager={1}'.format(
            self.data.__name__,
            self.manager.__name__)

        return '/'.join((self.get_news_context().absolute_url(),
                         '@@news_portlet_listing?{0}'.format(params)))

    def get_news_context(self):
        """ If we are in a news entry we have to get a parent node as start for
        our query. Else it would only display the current obj in the portlet.
        """
        if INews.providedBy(self.context):
            return aq_parent(aq_inner(self.context))
        return self.context


class EditForm(form.EditForm):
    implements(IPortletEditForm)
    label = _(u'Add News Portlet')
    description = _(u'This Portlet displays News')

    fields = field.Fields(INewsPortlet)

    def __init__(self, context, request):
        super(EditForm, self).__init__(context, request)
        self.status = None
        self._finishedAdd = None

    def __call__(self):
        IPortletPermissionChecker(aq_parent(aq_inner(self.context)))()
        return super(EditForm, self).__call__()

    def nextURL(self):
        editview = aq_parent(aq_inner(self.context))
        context = aq_parent(aq_inner(editview))
        url = str(getMultiAdapter((context, self.request),
                                  name=u"absolute_url"))
        return url + '/@@manage-portlets'

    @button.buttonAndHandler(_(u"label_save", default=u"Save"), name='apply')
    def handleSave(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        changes = self.applyChanges(data)
        if changes:
            self.status = "Changes saved"
        else:
            self.status = "No changes"

        nextURL = self.nextURL()
        return self.request.response.redirect(nextURL)

    @button.buttonAndHandler(_(u"label_cancel", default=u"Cancel"),
                             name='cancel_add')
    def handleCancel(self, action):
        nextURL = self.nextURL()
        return self.request.response.redirect(nextURL)

    def updateWidgets(self):
        self.fields['classification_items'].widgetFactory = \
            MultiContentTreeFieldWidget
        self.fields['path'].widgetFactory = MultiContentTreeFieldWidget
        if not self.context.portal_types.get('ClassificationItem', None):
            self.fields['classification_items'].mode = interfaces.HIDDEN_MODE

        super(EditForm, self).updateWidgets()



from AccessControl import ClassSecurityInfo
from ftw.contentpage import _
from ftw.contentpage.config import PROJECTNAME
from ftw.contentpage.content.schema import finalize
from ftw.contentpage.interfaces import IListingBlock
from plone.registry.interfaces import IRegistry
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata
from simplelayout.base.interfaces import ISimpleLayoutBlock
from zope.component import getUtility, queryMultiAdapter
from zope.interface import implements

from Products.ATContentTypes.config import HAS_LINGUA_PLONE
if HAS_LINGUA_PLONE:
    from Products.LinguaPlone import public as atapi
else:
    from Products.Archetypes import atapi


schema = atapi.Schema((
    atapi.BooleanField(
        'showTitle',
        schemata='default',
        default=True,
        widget=atapi.BooleanWidget(
        label=_(u'label_show_title',
                default=u'Show Title'))),

    atapi.LinesField(
        'tableColumns',
        schemata='default',
        required=True,
        default_method="getDefaultTableColumns",
        vocabulary='getColumns',
        widget=atapi.InAndOutWidget(
        label=_(u'Columns',
                default=u'Columns'))),

    atapi.StringField(
        'sortOn',
        required=True,
        default="sortable_title",
        vocabulary='getSortIndexVocabulary',
        widget=atapi.SelectionWidget(
            label=_(u'label_sort_on', default=u'Sort On'),
            description=_(u'help_sort_on', default=u''),
            format='select',
        ),
    ),

    atapi.StringField(
        'sortOrder',
        required=True,
        default="asc",
        vocabulary=[('ascending', _(u'label_ascending', default=u'Ascending')),
                    ('descending', _(u'label_descending',
                        default=u'Descending'))],
        widget=atapi.SelectionWidget(
            label=_(u'label_sort_order', default=u'Sort Order'),
            description=_(u'help_sort_order', default=u''),
            format='select',
        ),
    )

))

listing_block_schema = folder.ATFolderSchema.copy() + schema.copy()

schemata.finalizeATCTSchema(
    listing_block_schema,
    folderish=True,
    moveDiscussion=False,
)


listing_block_schema['title'].required = False
listing_block_schema['title'].default_method = 'getDefaultTitle'

finalize(listing_block_schema, hide=['description'])


class ListingBlock(folder.ATFolder):
    """A listing block for simplelayout"""
    implements(IListingBlock, ISimpleLayoutBlock,)

    meta_type = "ListingBlock"
    schema = listing_block_schema
    security = ClassSecurityInfo()

    security.declarePublic('showAddMenu')
    def showAddMenu(self):
        return False

    security.declarePublic('canSetDefaultPage')
    def canSetDefaultPage(self):
        return False

    security.declarePrivate('getDefaultTitle')
    def getDefaultTitle(self):
        registry = getUtility(IRegistry)
        return registry.get(
            'ftw.contentpage.listingblock.defaulttitle', None) or ''

    security.declarePrivate('getDefaultTableColumns')
    def getDefaultTableColumns(self):
        """ Returns the default table columns defined in registry.
        """
        registry = getUtility(IRegistry)
        return registry.get(
            'ftw.contentpage.listingblock.defaulttablecolumns',
            ('getContentType', 'Title', 'modified'))

    security.declarePrivate('getColumns')
    def getColumns(self):
        display_list = atapi.DisplayList()
        view = self.restrictedTraverse('@@block_view')
        for col in view.columns():
            display_list.add(col['column'], col['column_title'])
        return display_list

    security.declarePrivate('getSortIndexVocabulary')
    def getSortIndexVocabulary(self):
        view = queryMultiAdapter((self, self.REQUEST), name=u'block_view')
        voc = atapi.DisplayList()
        for col in view.columns():
            if 'sort_index' in col:
                voc.add(col['sort_index'], _(u'label_%s' % col['sort_index'],
                        default=col['sort_index']))
        voc.add('getObjPositionInParent', _(u'label_position_in_folder',
                default=u'Position in Folder'))
        return voc

atapi.registerType(ListingBlock, PROJECTNAME)



from ftw.builder import Builder
from ftw.builder import create
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from unittest import TestCase
import transaction


class TestTextblockTeaser(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):

        self.contentpage = create(Builder('content page')
                                  .titled('ContentPage'))
        self.textblock = create(Builder('text block')
                                .titled('TextBlock')
                                .having(showTitle='True',
                                        imageCaption='Image Caption',
                                        text="Lorem ipsum dolor sit amet.")
                                .with_dummy_content()
                                .within(self.contentpage))

    @browsing
    def test_image_is_not_clickable_by_default(self, browser):
        browser.open(self.contentpage)

        self.assertEquals(0,
                          len(browser.css('.textblock .sl-img-wrapper a')))

    @browsing
    def test_image_shows_colorbox_when_clickable_but_no_teaser_set(self, browser):
        self.textblock.setImageClickable(True)
        transaction.commit()
        browser.open(self.contentpage)

        img_link = browser.css('.textblock .sl-img-wrapper a').first_or_none
        self.assertTrue(img_link, "Image is does not have a link")

        self.assertEquals(img_link.attrib['href'],
                          'http://nohost/plone/contentpage/textblock/image')

    @browsing
    def test_internal_teaser(self, browser):
        linktarget = create(Builder('content page'))

        self.textblock.setTeaserSelectLink('intern')
        self.textblock.setTeaserReference(linktarget)
        transaction.commit()

        browser.open(self.textblock)
        self.assertEquals(
            'http://nohost/plone/contentpage-1',
            browser.css('.textblock .sl-img-wrapper a').first.attrib['href'])

    @browsing
    def test_external_teaser(self, browser):
        linktarget = 'http://www.google.ch'

        self.textblock.setTeaserSelectLink('extern')
        self.textblock.setTeaserExternalUrl(linktarget)
        transaction.commit()

        browser.open(self.contentpage)

        self.assertEquals(
            linktarget,
            browser.css('.textblock .sl-img-wrapper a').first.attrib['href'])

    @browsing
    def test_teaser_disables_colorbox(self, browser):
        linktarget = 'http://www.google.ch'

        # teaser takes priority over colorbox
        self.textblock.setImageClickable(True)

        self.textblock.setTeaserSelectLink('extern')
        self.textblock.setTeaserExternalUrl(linktarget)
        transaction.commit()

        browser.open(self.contentpage)

        self.assertEquals(
            ['disableColorbox'],
            browser.css('.textblock .sl-img-wrapper a').first.classes,
            "Teaser image needs marker class to not get colorboxed.")

        self.assertEquals(
            linktarget,
            browser.css('.textblock .sl-img-wrapper a').first.attrib['href'])

    @browsing
    def test_teaser_link_on_title(self, browser):
        linktarget = 'http://www.google.ch'

        browser.open(self.contentpage)
        self.assertEquals(
            0,
            len(browser.css('div.TextBlock > h2 > a')),
            "The title must not contain a link.")

        self.textblock.setTeaserSelectLink('extern')
        self.textblock.setTeaserExternalUrl(linktarget)
        transaction.commit()

        browser.open(self.contentpage)
        self.assertEquals(
            1,
            len(browser.css('div.TextBlock > h2 > a')),
            "The title should link to the teaser target."
            )

        self.assertEquals(
            'http://www.google.ch',
            browser.css('.TextBlock > h2 > a').first.attrib['href'],
            "The title must link to the teaser target.")



import os

from DateTime import DateTime
from ftw.builder import Builder, create
from ftw.testbrowser import browsing
from plone.app.testing import TEST_USER_NAME, TEST_USER_PASSWORD
from plone.portlets.interfaces import IPortletManager
from plone.testing.z2 import Browser
from Products.CMFCore.TypesTool import FactoryTypeInformation
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from pyquery import PyQuery
from zope.component import getUtility
import transaction
import unittest2 as unittest

from ftw.contentpage.browser.newslisting import NewsListing
from ftw.contentpage.portlets.news_archive_portlet import Assignment
from ftw.contentpage.portlets.news_archive_portlet import Renderer
from ftw.contentpage.portlets.news_portlet import Assignment as NewsAssignment
from ftw.contentpage.portlets.news_portlet import Renderer as NewsRenderer
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING


class TestNewsPortlets(unittest.TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def _create_portlet(self):
        self.browser.open(
            self.portal.absolute_url() +
            '/++contextportlets++plone.leftcolumn/+/newsportlet'
            )
        self.browser.getControl(
            name="form.widgets.portlet_title").value = u"My Portlet"

        self.browser.getControl(name="form.buttons.add").click()

    def _create_content(self):
        """
        Some of the test methods are using the same test data. But not all
        method need the test data. You may call this method at the top of
        your test method in order to get some test data.
        """
        self.newsfolder1 = self.portal.get(self.portal.invokeFactory(
            'NewsFolder', 'newsfolder1', title="Newsfolder1"))
        self.newsfolder2 = self.portal.get(self.portal.invokeFactory(
            'NewsFolder', 'newsfolder2', title="Newsfolder2"))
        self.newsfolder1.invokeFactory(
            'News',
            'news1',
            description="This Description must be longer than 50 chars"
            " so we are able to test if it will be croped",
            image=open("%s/dummy.png" % os.path.split(__file__)[0], 'r'),
            effectiveDate=DateTime() - 1)  # Yesterday
        self.newsfolder1.invokeFactory(
            'News', 'news2',
            effectiveDate=DateTime() - 15)  # Few days ago
        self.newsfolder2.invokeFactory(
            'News', 'news3',
            effectiveDate=DateTime() - 100)  # Old News
        self.newsfolder2.invokeFactory(
            'News', 'news4',
            effectiveDate=DateTime() + 5)  # Further news
        transaction.commit()

    def setUp(self):
        self.portal = self.layer['portal']
        self.browser = Browser(self.layer['portal'])
        self.browser.handleErrors = False
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD, ))

    def test_area_path_validator(self):
        self._create_content()
        self.browser.open(
            self.portal.absolute_url() +
            '/++contextportlets++plone.leftcolumn/+/newsportlet'
            )
        self.browser.getControl(
            name="form.widgets.portlet_title").value = u"My Portlet"
        # Get Control over the Query Field and enter a value.
        self.browser.getControl(
            name="form.widgets.path.widgets.query").value = u"ne"
        # Click the Searchbutton
        self.browser.getControl(
            name="form.widgets.path.buttons.search").click()
        # select the correct radio button over the Label.
        # Remember to use the Text of the Label and not the id. It won't work.
        self.browser.getControl("Newsfolder1").selected = True

        self.browser.getControl("label_only_context").selected = True
        self.browser.getControl(name="form.buttons.add").click()

        self.assertIn('You can not set a path and limit to context.',
                      self.browser.contents)

    def test_create_portlet_only_context(self):
        self._create_content()
        self.browser.open(
            self.portal.absolute_url() +
            '/++contextportlets++plone.leftcolumn/+/newsportlet'
            )
        self.browser.getControl(
            name="form.widgets.portlet_title").value = u"My Portlet"

        self.browser.getControl(name="form.buttons.add").click()
        self.browser.open(self.portal.absolute_url())

        self.assertIn('newsfolder1/news1', self.browser.contents)
        self.assertIn('newsfolder1/news2', self.browser.contents)
        self.assertIn('newsfolder2/news3', self.browser.contents)
        self.assertIn('newsfolder2/news4', self.browser.contents)
        self.browser.open(self.portal.newsfolder1.absolute_url())
        self.assertIn('newsfolder1/news1', self.browser.contents)
        self.assertIn('newsfolder1/news2', self.browser.contents)
        self.assertFalse('newsfolder2/news3' in self.browser.contents)
        self.assertFalse('newsfolder2/news4' in self.browser.contents)

    def test_create_portlet_path(self):
        self._create_content()
        self.browser.open(
            self.portal.absolute_url() +
            '/++contextportlets++plone.leftcolumn/+/newsportlet'
            )
        self.browser.getControl(
            name="form.widgets.portlet_title").value = u"My Portlet"
        # Get Control over the Query Field and enter a value.
        self.browser.getControl(
            name="form.widgets.path.widgets.query").value = u"ne"
        # Click the Searchbutton
        self.browser.getControl(
            name="form.widgets.path.buttons.search").click()
        # select the correct radio button over the Label.
        # Remember to use the Text of the Label and not the id. It won't work.
        self.browser.getControl("Newsfolder1").selected = True

        self.browser.getControl("label_only_context").selected = False
        self.browser.getControl(name="form.buttons.add").click()
        self.browser.open(self.portal.newsfolder1.absolute_url())
        self.assertIn('newsfolder1/news1', self.browser.contents)
        self.assertIn('newsfolder1/news2', self.browser.contents)
        self.assertFalse('newsfolder2/news3' in self.browser.contents)
        self.assertFalse('newsfolder2/news4' in self.browser.contents)

    def test_create_portlet_no_img(self):
        self._create_content()
        self.browser.open(
            self.portal.absolute_url() +
            '/++contextportlets++plone.leftcolumn/+/newsportlet'
            )
        self.browser.getControl(
            name="form.widgets.portlet_title").value = u"My Portlet"
        # Get Control over the Query Field and enter a value.
        self.browser.getControl(
            name="form.widgets.path.widgets.query").value = u"ne"
        # Click the Searchbutton
        self.browser.getControl(
            name="form.widgets.path.buttons.search").click()
        # select the correct radio button over the Label.
        # Remember to use the Text of the Label and not the id. It won't work.
        self.browser.getControl("Newsfolder1").selected = True
        self.browser.getControl("label_show_image").selected = False

        self.browser.getControl("label_only_context").selected = False
        self.browser.getControl(name="form.buttons.add").click()
        self.browser.open(self.portal.newsfolder1.absolute_url())
        self.assertNotIn('<div class="newsImage"', self.browser.contents)

    def test_create_portlet_crop_desc(self):
        self._create_content()
        self._create_portlet()
        self.browser.open(self.portal.absolute_url())
        self.assertIn('This Description must be longer than 50 chars so ...',
                      self.browser.contents)

    def test_create_portlet_desc_off(self):
        self._create_content()
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD, ))

        self.browser.open(
            self.portal.absolute_url() +
            '/++contextportlets++plone.leftcolumn/+/newsportlet')
        self.browser.getControl(
            name="form.widgets.portlet_title").value = u"My Portlet"
        # Get Control over the Query Field and enter a value.
        self.browser.getControl(
            name="form.widgets.path.widgets.query").value = u"ne"
        # Click the Searchbutton
        self.browser.getControl(
            name="form.widgets.path.buttons.search").click()
        # select the correct radio button over the Label.
        # Remember to use the Text of the Label and not the id. It won't work.
        self.browser.getControl(
            "Newsfolder1").selected = True

        self.browser.getControl("Show Description").selected = False
        self.browser.getControl(name="form.buttons.add").click()
        self.browser.open(self.portal.absolute_url())
        self.assertFalse('This Description' in self.browser.contents)

    def test_no_classificationItems(self):
        self.browser.open(
            self.portal.absolute_url() +
            '/++contextportlets++plone.leftcolumn/+/newsportlet')
        self.assertNotIn('<div class="contenttreeWidget"'
                         'id="form-widgets-classification_items-contenttree">',
                         self.browser.contents
                         )

    def test_classificationItems(self):
        types_tool = getToolByName(self.portal, 'portal_types')
        types_tool['ClassificationItem'] = FactoryTypeInformation(
            'ClassificationItem')
        transaction.commit()
        self.browser.open(
            self.portal.absolute_url() +
            '/++contextportlets++plone.leftcolumn/+/newsportlet'
            )
        self.assertIn('<div class="contenttreeWidget"'
                      ' id="form-widgets-classification_items-contenttree">',
                      self.browser.contents
                      )

    def test_editform_empty(self):
        self._create_portlet()
        self.browser.getLink("News Portlet").click()
        self.browser.getControl(name="form.buttons.apply").click()
        self.assertEqual(
            self.portal.absolute_url() + '/@@manage-portlets',
            self.browser.url
        )

    def test_editform_cancel(self):
        self._create_portlet()
        self.browser.getLink("News Portlet").click()
        self.browser.getControl(
            name="form.widgets.portlet_title").value = u"Not My Portlet"
        self.browser.getControl(
            name="form.buttons.cancel_add").click()
        self.assertEqual(
            self.portal.absolute_url() + '/@@manage-portlets',
            self.browser.url
        )
        self.browser.getLink("News Portlet").click()
        self.assertNotEqual(
            self.browser.getControl(name="form.widgets.portlet_title").value,
            "Not My Portlet"
        )

    def test_editform_success(self):
        self._create_portlet()
        self.browser.getLink("News Portlet").click()
        self.browser.getControl(
            name="form.widgets.portlet_title").value = u"My edited Portlet"
        self.browser.getControl(name="form.buttons.apply").click()
        self.assertEqual(self.portal.absolute_url() + '/@@manage-portlets',
                         self.browser.url)
        self.browser.getLink("News Portlet").click()
        self.assertEqual(
            self.browser.getControl(name="form.widgets.portlet_title").value,
            "My edited Portlet"
        )

    def test_editform_send_error(self):
        self._create_portlet()
        self.browser.getLink("News Portlet").click()
        self.browser.getControl(name="form.widgets.portlet_title").value = u""
        self.browser.getControl(name="form.buttons.apply").click()
        self.assertIn('<div class="error">Required input is missing.</div>',
                      self.browser.contents)

    def test_image_viewlet(self):
        self._create_content()
        self._create_portlet()
        self.browser.open(self.portal.absolute_url())
        self.assertIn('<div class="newsImage">'
                      '<img src="http://nohost/plone/'
                      'newsfolder1/news1/@@images/',
                      self.browser.contents)

    def test_addform_cancel(self):
        self.browser.open(
            self.portal.absolute_url() +
            '/++contextportlets++plone.leftcolumn/+/newsportlet'
        )
        self.browser.getControl(name="form.buttons.cancel_add").click()
        self.assertEqual(self.portal.absolute_url() + '/@@manage-portlets',
                         self.browser.url)

    def test_addform_send_error(self):
        self._create_content()
        self.browser.open(
            self.portal.absolute_url() +
            '/++contextportlets++plone.leftcolumn/+/newsportlet'
        )
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD, ))

        self.browser.open(
            self.portal.absolute_url() +
            '/++contextportlets++plone.leftcolumn/+/newsportlet'
        )
        # Get Control over the Query Field and enter a value.
        self.browser.getControl(
            name="form.widgets.path.widgets.query"
        ).value = u"ne"
        # Click the Searchbutton
        self.browser.getControl(
            name="form.widgets.path.buttons.search").click()
        self.browser.getControl("Newsfolder1").selected = True

        self.browser.getControl("label_only_context").selected = False
        self.browser.getControl(name="form.buttons.add").click()
        self.assertIn('<div class="error">Required input is missing.</div>',
                      self.browser.contents)

    def test_days_no_filter(self):
        self._create_content()
        context = self.portal
        portlet = NewsAssignment(days=0, only_context=False)
        manager = getUtility(IPortletManager, name=u"plone.leftcolumn")
        renderer = NewsRenderer(context, context.REQUEST, BrowserView,
                                manager, portlet)
        self.assertEquals(len(renderer.get_news()), 4, 'Expect all 4 news')

    def test_days_filter(self):
        self._create_content()
        context = self.portal
        portlet = NewsAssignment(days=5, only_context=False)
        manager = getUtility(IPortletManager, name=u"plone.leftcolumn")
        renderer = NewsRenderer(context, context.REQUEST, BrowserView,
                                manager, portlet)

        self.assertEquals(len(renderer.get_news()), 2, 'Expect 2 news')

        portlet = NewsAssignment(days=28, only_context=False)
        renderer = NewsRenderer(context, context.REQUEST, BrowserView,
                                manager, portlet)
        self.assertEquals(len(renderer.get_news()), 3, 'Expect 3 news')

    def test_archive_portlet_empty(self):
        # Separate test for the archive portlet, because we need static dates
        archivefolder = self.portal.get(self.portal.invokeFactory(
            'NewsFolder', 'archivefolder', title="Archive Test"))

        manager = getUtility(IPortletManager, name=u"plone.leftcolumn")
        portlet = Assignment()
        view = NewsListing(archivefolder, archivefolder.REQUEST)
        renderer = Renderer(archivefolder, archivefolder.REQUEST, view,
                            manager, portlet)

        self.assertFalse(renderer.available,
                         'Did no expect an archive portlet')

    def test_archive_portlet(self):
        manager = getUtility(IPortletManager, name=u"plone.leftcolumn")

        archivefolder = self.portal.get(self.portal.invokeFactory(
            'NewsFolder', 'archivefolder', title="Archive Test"))

        archivefolder.invokeFactory(
            'News', 'news1', effectiveDate=DateTime('2013/01/20'))
        archivefolder.invokeFactory(
            'News', 'news2',
            effectiveDate=DateTime('2013/01/21'))
        archivefolder.invokeFactory(
            'News', 'news3', effectiveDate=DateTime('2012/12/20'))
        archivefolder.invokeFactory(
            'News', 'news4', effectiveDate=DateTime('2012/12/21'))
        archivefolder.invokeFactory(
            'News', 'news5', effectiveDate=None)

        portlet = Assignment()
        view = NewsListing(archivefolder, archivefolder.REQUEST)
        renderer = Renderer(archivefolder, archivefolder.REQUEST, view,
                            manager, portlet)

        self.assertTrue(renderer.available,
                        'There should be an archive portlet')

        self.assertEquals(
            [{'mark': False,
              'months': [{'mark': False,
                          'number': 2,
                          'title': u'January',
                          'url': 'http://nohost/plone/archivefolder/'
                                 'news_listing?archive=2013/01/01'}],
              'number': 2,
              'title': '2013'},
             {'mark': False,
              'months': [{'mark': False,
                          'number': 2,
                          'title': u'December',
                          'url': 'http://nohost/plone/archivefolder/'
                                 'news_listing?archive=2012/12/01'}],
              'number': 2,
              'title': '2012'}],
            renderer.archive_summary())

    def test_archive_portlets_is_available_on_newslisting(self):
        self._create_content()
        manager = getUtility(IPortletManager, name=u"plone.leftcolumn")
        portlet = Assignment()
        view = NewsListing(self.portal, self.portal.REQUEST)
        renderer = Renderer(self.portal, self.portal.REQUEST, view,
                            manager, portlet)

        self.assertTrue(
            renderer.available,
            'News Archive portlet should be available on NewsListing View'
        )

    def test_news_archive_portlet(self):
        self._create_content()
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD, ))

        self.browser.open(
            '%s/++contextportlets++plone.leftcolumn/+/newsarchiveportlet' %
            self.newsfolder1.absolute_url()
        )

        self.browser.open(self.newsfolder1.absolute_url())

        pq = PyQuery(self.browser.contents)
        self.assertTrue(
            pq('.portlet.portletArchiveListing'),
            'We added one, so there sould be a EventArchive portlet'
        )

        self.assertGreater(
            len(pq('.portlet.portletArchiveListing .portletItem li')), 0,
            'Expect at least one entry in the events archive portlet')

    def test_newsportlet_more_news_link_disabled(self):
        manager = getUtility(IPortletManager, name=u"plone.leftcolumn")
        portlet = NewsAssignment()
        renderer = NewsRenderer(self.portal, self.portal.REQUEST, object(),
                                manager, portlet)

        self.assertFalse(
            renderer.show_more_news_link(),
            'Expect that the "More News" link is invisible'
        )

    def test_newsportlet_more_news_link_enabled(self):
        manager = getUtility(IPortletManager, name=u"plone.leftcolumn")
        portlet = NewsAssignment(more_news_link=True)
        renderer = NewsRenderer(self.portal, self.portal.REQUEST, object(),
                                manager, portlet)

        self.assertTrue(
            renderer.show_more_news_link(),
            'Expect that the "More News" link is visible'
        )

    def test_newsportets_does_not_show_rss_link(self):
        manager = getUtility(IPortletManager, name=u"plone.leftcolumn")
        portlet = NewsAssignment(rss_link=False)
        renderer = NewsRenderer(self.portal, self.portal.REQUEST, object(),
                                manager, portlet)

        self.assertFalse(renderer.show_rss_link(),
                         'show_rss_link is not enabled.')

        doc = PyQuery(renderer.render())
        self.assertFalse(doc('.RssLink'),
                         'There should be no link in the portlet')

    def test_newsportets_shows_rss_link(self):
        manager = getUtility(IPortletManager, name=u"plone.leftcolumn")
        portlet = NewsAssignment(rss_link=True)
        renderer = NewsRenderer(self.portal, self.portal.REQUEST, object(),
                                manager, portlet)

        self.assertTrue(renderer.show_rss_link(), 'show_rss_link is enabled')

        doc = PyQuery(renderer.render())
        self.assertTrue(
            doc('.RssLink'),
            'There should be a RSS link in the portlet'
        )

    @browsing
    def test_news_portlet_is_not_available_without_news_entries(self, browser):
        folder = create(Builder('news folder').titled('News'))

        create(Builder('news portlet')
               .having(only_context=False,
                       path=['/{0}'.format(folder.getId())],
                       more_news_link=True))

        browser.login().open()
        self.assertEquals(None, browser.find('More News'))

    @browsing
    def test_news_portlet_is_available_without_news_entries(self, browser):
        folder = create(Builder('news folder').titled('News'))

        create(Builder('news portlet')
               .having(always_render_portlet=True,
                       only_context=False,
                       path=['/{0}'.format(folder.getId())],
                       more_news_link=True))

        browser.login().open()
        self.assertNotEquals(None, browser.find('More News'))

    @browsing
    def test_news_portlet_does_not_render_expired_items(self, browser):
        news_folder = create(Builder('news folder'))

        # Create six news items.
        create(Builder('news')
               .titled('Default News')
               .within(news_folder))
        create(Builder('news')
               .titled('Default News With Expiration Date')
               .within(news_folder)
               .having(expirationDate=DateTime() + 20))
        create(Builder('news')
               .titled('Future News')
               .within(news_folder)
               .having(effectiveDate=DateTime() + 10))
        create(Builder('news')
               .titled('Future News With Expiration Date')
               .within(news_folder)
               .having(effectiveDate=DateTime() + 10)
               .having(expirationDate=DateTime() + 20))
        create(Builder('news')
               .titled('Old News')
               .within(news_folder)
               .having(effectiveDate=DateTime() - 10))
        create(Builder('news')
               .titled('Old News (Expired)')
               .within(news_folder)
               .having(effectiveDate=DateTime() - 20)
               .having(expirationDate=DateTime() - 10))

        create(Builder('news portlet')
               .in_manager(u'plone.rightcolumn')
               .within(self.portal)
               .having(portlet_title='My Portlet',
                       more_news_link=True,
                       quantity=100))

        # Make sure the admin sees all news entries, even expired and future
        # news items.
        browser.login().open()
        self.assertEqual(
            [
                'Future News',
                'Future News With Expiration Date',
                'Default News',
                'Default News With Expiration Date',
                'Old News',
                'Old News (Expired)',
            ],
            browser.css('.newsText .portletItemTitle').text
        )

        # Make sure the anonymous user only sees news entries which are
        # not expired and not in the future.
        browser.logout().open()
        self.assertEqual(
            [
                'Default News With Expiration Date',
                'Default News',
                'Old News',
            ],
            browser.css('.newsText .portletItemTitle').text
        )

        # Configure the portlet to show expired news too.
        browser.login()
        browser.visit(self.portal, view='manage-portlets')
        browser.find('News Portlet (My Portlet)').click()
        browser.fill({u'Show expired items': True}).submit()
        browser.forms['form'].fill({u'Show expired items': True}).find('Save').click()

        # Make sure the admin still sees all news entries, even expired
        # and future news items.
        browser.login().open()
        self.assertEqual(
            [
                'Future News',
                'Future News With Expiration Date',
                'Default News',
                'Default News With Expiration Date',
                'Old News',
                'Old News (Expired)',
            ],
            browser.css('.newsText .portletItemTitle').text
        )

        # Make sure the anonymous user still only sees news entries which are
        # not expired and not in the future.
        browser.logout().open()
        self.assertEqual(
            [
                'Default News With Expiration Date',
                'Default News',
                'Old News',
            ],
            browser.css('.newsText .portletItemTitle').text
        )



from ftw.upgrade import UpgradeStep


class UpdateRolemapAndFactoryTool(UpgradeStep):
    """Update rolemap and factory tool
    """

    def __call__(self):
        self.setup_install_profile(
            'profile-ftw.contentpage.upgrades:1006')



from ftw.upgrade import UpgradeStep


class UpdateViews(UpgradeStep):
    """Add simplelayout_summary view
    """

    def __call__(self):
        self.setup_install_profile(
            'profile-ftw.contentpage.upgrades:1400')



from ftw.upgrade import UpgradeStep


class RemoveWorkflowFromBlocks(UpgradeStep):
    """Remove Workflow from TextBlock, ListingBlock, AddressBlock
    """

    def __call__(self):
        self.setup_install_profile(
            'profile-ftw.contentpage.upgrades:1008')

        self.remove_security_settings_from_blocks()

    def remove_security_settings_from_blocks(self):
        query = {'portal_type': ['ListingBlock', 'AddressBlock', 'TextBlock']}
        for obj in self.catalog_unrestricted_search(query, full_objects=True):
            for item in obj.ac_inherited_permissions(1):
                permission = item[0]
                obj.manage_permission(permission, roles=[], acquire=True)
                obj.reindexObjectSecurity()



from ftw.testing import MockTestCase
from ftw.contentpage.vocabularies import SubjectVocabulary
from plone.i18n.normalizer.interfaces import IIDNormalizer
from mocker import ANY


class TestVocabulary(MockTestCase):

    def setUp(self):
        super(TestVocabulary, self).setUp()
        self.context = self.stub()
        self.cat = self.stub()
        self.mock_tool(self.cat, 'portal_catalog')
        normalizer = self.stub()
        self.mock_utility(normalizer, IIDNormalizer)
        self.expect(normalizer.normalize(ANY)).call(lambda term: term.lower())

    def set_subjects(self, subjects=[]):
        self.expect(self.cat.uniqueValuesFor("Subject")).result(subjects)
        self.replay()

    def test_vocabulary_needs_titles_in_unicode(self):
        self.set_subjects(['J\xc3\xa4m', 'Cheese'])

        self.assertEqual(
            [u'J\xe4m', u'Cheese'],
            [item.title for item in SubjectVocabulary(self.context)]
        )

    def test_vocabulary_values(self):
        self.set_subjects(['J\xc3\xa4m', 'Cheese'])

        self.assertEqual(
            ['J\xc3\xa4m', 'Cheese'],
            [item.value for item in SubjectVocabulary(self.context)]
        )

    def test_vocabulary_needs_unique_tokens(self):
        self.set_subjects(['Cheese', 'CHEESE'])

        self.assertEquals(
            [
            'a67778b3dcc82bfaace0f8bc0061f20e-cheese',
            '0b2ebce559f2697d6be386d52840f087-cheese',
            ],
            [item.token for item in SubjectVocabulary(self.context)]
        )



from ftw.contentpage.testing import ZCML_LAYER
from ftw.testing import MockTestCase
from zope.component import getMultiAdapter
from ftw.contentpage.interfaces import IAddressBlock
from simplelayout.base.interfaces import ISimpleLayoutCapable
from ftw.contentpage.interfaces import IFtwContentPageLayer
from mocker import ANY


class TestAddressBlockView(MockTestCase):

    layer = ZCML_LAYER

    def setUp(self):
        super(TestAddressBlockView, self).setUp()

        self.request = self.stub_request(IFtwContentPageLayer)
        self.context = self.providing_stub(IAddressBlock)
        self.expect(self.context.getAddress()).result("Address line")
        self.expect(self.context.getExtraAddressLine()).result(
            "Additional line")

    def test_component_registered_block_view(self):
        self.replay()
        self.assertNotEquals(getMultiAdapter(
            (self.context, self.request), name="block_view"),
            None)

    def test_component_registered_portlet_view(self):
        self.replay()
        self.assertNotEquals(getMultiAdapter(
            (self.context, self.request), name="block_view-portlet"),
            None)

    def test_component_registered_detail_view(self):
        self.replay()
        self.assertNotEquals(getMultiAdapter(
            (self.context, self.request), name="addressblock_detail_view"),
            None)

    def test_get_address_as_html(self):
        self.replay()
        view = getMultiAdapter((self.context, self.request),
                               name="block_view")
        self.assertEquals(view.get_address_as_html(),
                          "Address line<br />Additional line")

    def test_has_team(self):
        parent = self.providing_stub(ISimpleLayoutCapable)
        self.set_parent(self.context, parent)
        self.expect(parent.getFolderContents(contentFilter=ANY)).result(True)
        self.replay()
        view = getMultiAdapter(
            (self.context, self.request), name="block_view-portlet")
        self.assertTrue(view.has_team())

    def test_has_no_team(self):
        parent = self.providing_stub(ISimpleLayoutCapable)
        self.set_parent(self.context, parent)
        self.expect(parent.getFolderContents(contentFilter=ANY)).result(
            False)
        self.replay()
        view = getMultiAdapter(
            (self.context, self.request), name="block_view-portlet")
        self.assertFalse(view.has_team())



from ftw.contentpage.interfaces import IAuthority
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView


def get_brain_data(brain):
    return {'title': brain.Title,
            'path': brain.getPath(),
            'url': brain.getURL(),
            'children': []}


def get_parent_path(path):
    return '/'.join(path.rstrip('/').split('/')[:-1])


def make_treeish(data):
    path2node = dict(map(lambda item: (item['path'], item), data))

    tree = []
    for node in data:
        parent_path = get_parent_path(node['path'])
        if parent_path in path2node:
            path2node[parent_path]['children'].append(node)
        else:
            tree.append(node)

    return tree


class AuthoritiesView(BrowserView):

    sort_on = 'getObjPositionInParent'

    def columns(self):
        contents = self.contents()
        half = len(contents) / 2
        num_leftcolumn = getattr(self.context, 'leftcolumn_num_elements', half)

        columns = [{'classes': 'listing-column listing-column-left',
                    'nodes': contents[:num_leftcolumn]},
                   {'classes': 'listing-column listing-column-right',
                    'nodes': contents[num_leftcolumn:]}]
        return columns

    def contents(self):
        brains = self._get_brains()
        brains = filter(lambda brain: not brain.exclude_from_nav, brains)
        data = map(get_brain_data, brains)
        data = make_treeish(data)
        return data

    def get_query(self):
        path = '/'.join(self.context.getPhysicalPath())
        return {'object_provides': IAuthority.__identifier__,
                'sort_on': self.sort_on,
                'path': {'query': path, 'depth': 2}}

    def _get_brains(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        return catalog(self.get_query())



from ftw.builder import Builder
from ftw.builder import create
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from ftw.testbrowser.pages import statusmessages
from simplelayout.base.interfaces import ISimpleLayoutBlock
from unittest2 import TestCase


class TestTextBlockCreation(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        self.contentpage = create(Builder('content page').titled('The Page'))

    def test_simplelayout_integration(self):
        textblock = create(Builder('text block').within(self.contentpage))
        ISimpleLayoutBlock.providedBy(textblock)

    @browsing
    def test_text_is_used_as_title(self, browser):
        browser.login().open(self.contentpage)
        factoriesmenu.add('TextBlock')
        browser.fill({'Text': '<p>Hello World</p>'}).save()
        statusmessages.assert_no_error_messages()
        block = self.contentpage.objectValues()[0]
        self.assertEquals('Hello World', block.Title())

    @browsing
    def test_title_is_cropped_chars_when_filled_with_text(self, browser):
        browser.login().open(self.contentpage)
        factoriesmenu.add('TextBlock')
        browser.fill({'Text': '<p>%s</p>' % ('X' * 50)}).save()
        statusmessages.assert_no_error_messages()
        block = self.contentpage.objectValues()[0]
        self.assertEquals(29, len(block.Title()))

    @browsing
    def test_alt_text_is_image_filename_when_empty(self, browser):
        create(Builder('text block')
               .within(self.contentpage)
               .with_dummy_content()
               .having(text='<p>Text</p>'))
        browser.login().open(self.contentpage)
        self.assertEqual(
            'image.gif',
            browser.css('.sl-img-wrapper img').first.attrib['alt'])



import unittest2 as unittest
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
import transaction
from simplelayout.base.interfaces import ISimpleLayoutCapable
from plone.testing.z2 import Browser
from plone.app.testing import TEST_USER_NAME, TEST_USER_PASSWORD
from StringIO import StringIO
import os

class TestNews(unittest.TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.newsfolder = self.portal.get(self.portal.invokeFactory('NewsFolder', 'newsfolder'))
        transaction.commit()
        self.browser = Browser(self.layer['app'])

    def test_creation_without_img(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
                TEST_USER_NAME, TEST_USER_PASSWORD, ))
        self.browser.open(self.newsfolder.absolute_url()+'/createObject?type_name=News')
        self.browser.getControl(name="title").value = "My News"
        self.browser.getControl(name="form.button.save").click()
        self.assertEqual(self.browser.url, 'http://nohost/plone/newsfolder/my-news/view')

    def test_creation_with_img(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
                TEST_USER_NAME, TEST_USER_PASSWORD, ))
        self.browser.open(self.newsfolder.absolute_url()+'/createObject?type_name=News')
        self.browser.getControl(name="title").value = "My News"
        file_ = open("%s/dummy.png" % os.path.split(__file__)[0])
        file_field = self.browser.getControl(name="image_file")
        file_field.add_file(StringIO(file_.read()), 'image/png', 'dummy.png')
        self.browser.getControl(name="form.button.save").click()
        self.assertIn('sl-img-wrapper', self.browser.contents)

    def test_interfaces(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
                TEST_USER_NAME, TEST_USER_PASSWORD, ))
        self.browser.open(self.newsfolder.absolute_url()+'/createObject?type_name=News')
        self.browser.getControl(name="title").value = "My News"
        self.browser.getControl(name="form.button.save").click()
        news = self.newsfolder.get("my-news")
        self.assertTrue(ISimpleLayoutCapable.providedBy(news))



from ftw.upgrade import UpgradeStep


class UpdateRegistry(UpgradeStep):
    """Add alphabetical subject listing configuration to registry.
    """

    def __call__(self):
        self.setup_install_profile(
            'profile-ftw.contentpage.upgrades:1401')



from plone.app.layout.viewlets.common import ViewletBase
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class OpenlayersViewlet(ViewletBase):
    index = ViewPageTemplateFile('openlayers.pt')

    def available(self):
        return bool(self.context.getFolderContents({
            'portal_type': 'AddressBlock'}))



from ftw.upgrade import UpgradeStep


class AddRoleMapEntrys(UpgradeStep):
    """Add missing rolemap entrys for the add permissions.
    """

    def __call__(self):
        self.setup_install_profile(
            'profile-ftw.contentpage.upgrades:1005')



from ftw.contentpage.interfaces import INewsListingView
from ftw.contentpage.portlets.base_archive_portlet import ArchiveSummary
from plone.app.portlets.portlets import base
from plone.memoize.view import memoize
from plone.portlets.interfaces import IPortletDataProvider
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.interface import implements


class INewsArchivePortlet(IPortletDataProvider):
    """Archive portlet interface.
"""


class Assignment(base.Assignment):
    implements(INewsArchivePortlet)

    @property
    def title(self):
        return "News Archive Portlet"


class Renderer(base.Renderer):
    def __init__(self, context, request, view, manager, data):
        self.context = context
        self.data = data
        self.request = request
        self.view = view

    @property
    def available(self):
        """Only show the portlet:
        - If there are News
        - If view is NewsListing
        """
        has_news = bool(self.archive_summary())

        if INewsListingView.providedBy(self.view):
            return has_news
        else:
            return False

    @memoize
    def archive_summary(self):
        """Returns an ordered list of summary infos per month."""
        return ArchiveSummary(
            self.context,
            self.request,
            ['ftw.contentpage.interfaces.INews'],
            'effective',
            'news_listing')()

    render = ViewPageTemplateFile('news_archive_portlet.pt')


class AddForm(base.NullAddForm):

    def create(self):
        return Assignment()



from AccessControl import ClassSecurityInfo
from ftw.contentpage.config import PROJECTNAME
from ftw.contentpage.interfaces import IEventFolder
from Products.ATContentTypes.config import HAS_LINGUA_PLONE
from Products.ATContentTypes.content import folder
from simplelayout.base.interfaces import ISimpleLayoutCapable
from zope.interface import implements


if HAS_LINGUA_PLONE:
    from Products.LinguaPlone.public import registerType
else:
    from Products.Archetypes.atapi import registerType


class EventFolder(folder.ATFolder):

    implements(IEventFolder, ISimpleLayoutCapable)
    security = ClassSecurityInfo()


registerType(EventFolder, PROJECTNAME)



from ftw.testing import MockTestCase
from ftw.contentpage.portlets.news_portlet import Assignment, Renderer
from mocker import MATCH


class TestNewsPortlet(MockTestCase):

    def test_onlycontext(self):

        def query_matcher(query):
            self.assertEqual(query['path']['query'], 'folder/file')
            return True

        context = self.mocker.mock()
        request = self.mocker.mock()
        view = self.mocker.mock()
        manager = self.mocker.mock()
        catalog = self.mocker.mock()
        url = self.mocker.mock()
        self.mock_tool(catalog, 'portal_catalog')
        self.mock_tool(url, 'portal_url')
        self.expect(url.getPortalPath()).result('/plone')
        self.expect(context.getPhysicalPath()).result(('folder', 'file'))
        self.expect(catalog.searchResults(
            MATCH(query_matcher), show_all=False, show_inactive=False)).count(1).result(['1', '2', '3', '4', '5'])
        self.replay()
        obj = Assignment(path=['/peter/hans'])
        renderer = Renderer(context, request, view, manager, obj)
        renderer.get_news()

    def test_path(self):

        def query_matcher(query):
            self.assertEqual(query['path']['query'], ['/plone/peter/hans'])
            return True

        context = self.mocker.mock()
        request = self.mocker.mock()
        view = self.mocker.mock()
        manager = self.mocker.mock()
        catalog = self.mocker.mock()
        url = self.mocker.mock()
        self.mock_tool(catalog, 'portal_catalog')
        self.mock_tool(url, 'portal_url')
        self.expect(url.getPortalPath()).result('/plone')
        self.expect(context.getPhysicalPath()).result(
            ('folder', 'file')).count(0, None)
        self.expect(catalog.searchResults(
            MATCH(query_matcher), show_all=False, show_inactive=False)).count(1).result(['1', '2', '3', '4', '5'])
        self.replay()
        obj = Assignment(path=['peter/hans'], only_context=False)
        renderer = Renderer(context, request, view, manager, obj)
        renderer.get_news()

    def test_subject(self):

        def query_matcher(query):
            self.assertEqual(query['Subject'], ['hans', 'peter'])
            return True

        context = self.mocker.mock()
        request = self.mocker.mock()
        view = self.mocker.mock()
        manager = self.mocker.mock()
        catalog = self.mocker.mock()
        url = self.mocker.mock()
        self.mock_tool(catalog, 'portal_catalog')
        self.mock_tool(url, 'portal_url')
        self.expect(url.getPortalPath()).result('/plone')
        self.expect(context.getPhysicalPath()).result(
            ('folder', 'file')).count(0, None)
        self.expect(catalog.searchResults(
            MATCH(query_matcher), show_all=False, show_inactive=False)).count(1).result(['1', '2', '3', '4', '5'])
        self.replay()
        obj = Assignment(subjects=['hans', 'peter'], only_context=False)
        renderer = Renderer(context, request, view, manager, obj)
        renderer.get_news()

    def test_ClassificationItem(self):

        def query_matcher(query):
            self.assertEqual(query['cs_uids'], ['1234567'])
            return True

        context = self.mocker.mock()
        request = self.mocker.mock()
        view = self.mocker.mock()
        manager = self.mocker.mock()
        catalog = self.mocker.mock()
        url = self.mocker.mock()
        obj_uid = self.mocker.mock()
        self.mock_tool(catalog, 'portal_catalog')
        self.mock_tool(url, 'portal_url')
        self.expect(obj_uid.UID()).result('1234567')
        self.expect(url.getPortalPath()).result('/plone')
        self.expect(context.restrictedTraverse(
            '/plone/hans/peter')).result(obj_uid)
        self.expect(context.getPhysicalPath()).result(
            ('folder', 'file')).count(0, None)
        self.expect(catalog.searchResults(
            MATCH(query_matcher), show_all=False, show_inactive=False)).count(1).result(['1', '2', '3', '4', '5'])
        self.replay()
        obj = Assignment(classification_items=['hans/peter'],
                         only_context=False)
        renderer = Renderer(context, request, view, manager, obj)
        renderer.get_news()



from Acquisition import aq_inner
from email.header import Header
from email.mime.text import MIMEText
from ftw.contentpage import _
from plone import api
from plone.formwidget.recaptcha.widget import ReCaptchaFieldWidget
from plone.registry.interfaces import IRegistry
from plone.z3cform.layout import wrap_form
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import form, field, button
from z3c.form.interfaces import WidgetActionExecutionError
from zope import schema
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import Interface
from zope.interface import Invalid
import re


def is_email(value):
    expr = re.compile(r"^(\w&.%#$&'\*+-/=?^_`{}|~]+!)*[\w&.%#$&'\*+-/=" +
                      r"?^_`{}|~]+@(([0-9a-z]([0-9a-z-]*[0-9a-z])?" +
                      r"\.)+[a-z]{2,6}|([0-9]{1,3}\.){3}[0-9]{1,3})$",
                      re.IGNORECASE)

    return bool(expr.match(value))


class IFeedbackView(Interface):

    """Interface for z3c.form"""
    sender = schema.TextLine(title=_(u"Sender_Name", default=u"Name"),
                             required=True)
    email = schema.TextLine(title=_(u"mail_address", default="E-Mail"),
                            required=True,
                            constraint=is_email)
    subject = schema.TextLine(title=_(u"label_subject", default="Subject"),
                              required=True)
    message = schema.Text(title=_(u"label_message", default="Message"),
                          required=True)
    captcha = schema.TextLine(title=u"ReCaptcha",
                              required=False)


class FeedbackForm(form.Form):
    label = _(u"label_send_feedback", default=u"Send Feedback")
    fields = field.Fields(IFeedbackView)

    # don't use context to get widget data
    ignoreContext = True

    def updateWidgets(self):
        captcha_enabled = self.recaptcha_enabled()
        if captcha_enabled:
            self.fields['captcha'].widgetFactory = ReCaptchaFieldWidget

        super(FeedbackForm, self).updateWidgets()

        if not captcha_enabled:
            # Simply delete the widget instead of trying to set hidden mode,
            # which caused a `ComponentLookupError` in a special case. Please
            # see `test_captcha_no_component_lookup_error`.
            del self.widgets['captcha']

    def recaptcha_enabled(self):
        registry = getUtility(IRegistry, context=self)
        private_key = registry.get(
            'plone.formwidget.recaptcha.interfaces.'
            'IReCaptchaSettings.private_key',
            u'')
        public_key = registry.get(
            'plone.formwidget.recaptcha.interfaces.'
            'IReCaptchaSettings.public_key',
            u'')
        return (public_key and private_key and api.user.is_anonymous())

    @button.buttonAndHandler(_(u'Send Mail'))
    def handleApply(self, action):
        data, errors = self.extractData()

        if self.recaptcha_enabled():
            captcha = getMultiAdapter((aq_inner(self.context), self.request),
                                      name='recaptcha')
            if not captcha.verify():
                raise WidgetActionExecutionError(
                    'captcha',
                    Invalid(_("The captcha code you entered was wrong, "
                              "please enter the new one.")))

        if errors:
            return

        message = data.get('message')
        email = data.get('email')
        subject = data.get('subject')
        sender = data.get('sender').replace(',', ' ')
        self.send_feedback(email, subject, message, sender)
        msg = _(u'info_email_sent', default=u'The email was sent.')
        IStatusMessage(self.request).addStatusMessage(msg, type='info')
        return self.redirect()

    @button.buttonAndHandler(_(u'button_cancel', default=u'Cancel'))
    def handle_cancel(self, action):
        return self.redirect()

    def redirect(self):
        """Redirect back
        """
        url = self.context.aq_parent.absolute_url()
        return self.request.RESPONSE.redirect(url)

    def send_feedback(self, recipient, subject, message, sender):
        """Send a feedback email to the email address defined in
        the addressblock.
        """
        mh = getToolByName(self.context, 'MailHost')
        portal = getToolByName(self.context, 'portal_url').getPortalObject()

        text = translate(
            u'feedback_mail_text',
            domain='ftw.contentpage',
            default='${sender} sends you a message:\n${msg}',
            context=self.request,
            mapping={'sender': "%s (%s)" % (sender, recipient),
                     'msg': message})

        # create the message root with from, to, and subject headers
        msg = MIMEText(text.encode('utf-8'), 'plain', 'utf-8')
        msg['Subject'] = Header(subject, 'utf-8')

        msg['From'] = Header('%s' % portal.getProperty('email_from_name'),
                             'utf-8')
        msg['From'].append(
            "<%s>" % portal.getProperty('email_from_address').decode('utf-8'))
        if isinstance(sender, unicode):
            sender = sender.encode('utf-8')
        msg['Reply-To'] = Header("%s" % sender, 'utf-8')
        msg['Reply-To'].append("<%s>" % recipient)
        msg['To'] = self.context.getEmail()
        # send the message
        mh.send(msg)


FeedbackView = wrap_form(FeedbackForm)



from pkg_resources import get_distribution


PROJECTNAME = 'ftw.contentpage'


ADD_PERMISSIONS = {
    'ContentPage': 'ftw.contentpage: Add ContentPage',
    'AddressBlock': 'ftw.contentpage: Add AddressBlock',
    'ListingBlock': 'ftw.contentpage: Add ListingBlock',
    'TextBlock': 'ftw.contentpage: Add TextBlock',
    'News': 'ftw.contentpage: Add News',
    'NewsFolder': 'ftw.contentpage: Add NewsFolder',
    'EventPage': 'ftw.contentpage: Add EventPage',
    'EventFolder': 'ftw.contentpage: Add EventFolder',
    }

INDEXES = (('getContentCategories', 'KeywordIndex'),
          ('getContentType', 'FieldIndex'))


ORIGINAL_SIZE = (900, 900)



if get_distribution('Plone').version >= '4.3':
    HAS_CONTENT_LISTING_BEHAVIOR = True
else:
    HAS_CONTENT_LISTING_BEHAVIOR = False



from Products.TinyMCE.browser.atanchors import ATAnchorView


class ContentPageAnchorView(ATAnchorView):

    def listAnchorNames(self, *args, **kwargs):
        query = {'object_provides': 'simplelayout.base.interfaces.'
                                    'ISimpleLayoutBlock'}

        anchors = []
        for block in self.context.listFolderContents(contentFilter=query):
            view = block.restrictedTraverse('content_anchors')
            anchors.extend(view.listAnchorNames(*args, **kwargs))

        return anchors



from collective.geo.geographer.interfaces import IGeoreferenced
from ftw.builder import Builder
from ftw.builder import create
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.registry import field
from plone.registry import Record
from plone.registry.interfaces import IRegistry
from plone.testing.z2 import Browser
from simplelayout.base.interfaces import ISimpleLayoutBlock
from unittest2 import TestCase
from zope.component import getUtility
import transaction


class TestAddressBlockCreation(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestAddressBlockCreation, self).setUp()
        self.portal = self.layer['portal']
        self.portal_url = self.portal.portal_url()

        self.contentpage = self.portal.get(
            self.portal.invokeFactory('ContentPage', 'contentpage'))
        # Fire all necessary events
        self.contentpage.processForm()
        self.contentpage.reindexObject()
        transaction.commit()

        # Browser setup
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

    def _auth(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD, ))

    def _create_addressblock(self):
        addressblock = self.contentpage.get(
            self.contentpage.invokeFactory('AddressBlock', 'addressblock'))
        # Fire all necessary events
        addressblock.processForm()
        addressblock.reindexObject()
        transaction.commit()
        return addressblock

    def test_fti(self):
        self.assertIn('AddressBlock', self.portal.portal_types.objectIds())

    def test_creation(self):
        _id = self.contentpage.invokeFactory('AddressBlock', 'addressblock')
        self.assertIn(_id, self.contentpage.objectIds())

    def test_simplelayout_integration(self):
        addressblock = self.portal.get(
            self.contentpage.invokeFactory('AddressBlock', 'addressblock'))
        ISimpleLayoutBlock.providedBy(addressblock)

    def test_addressblock_view(self):
        addressblock = self._create_addressblock()
        self._auth()
        self.browser.open(self.contentpage.absolute_url())
        self.assertIn(addressblock.getId(), self.browser.contents)
        self.assertIn('simplelayout-block-wrapper AddressBlock',
                      self.browser.contents)

    def test_addressblock_default_title(self):
        addressblock = self._create_addressblock()
        self.assertEquals('Address', addressblock.Title())

        self._auth()
        self.browser.open(self.contentpage.absolute_url())
        self.assertIn('<h2>Address', self.browser.contents)

    def test_addressblock_change_default_title(self):
        registry = getUtility(IRegistry)
        registry.records['ftw.contentpage.addressblock.defaulttitle'] = \
            Record(field.TextLine(title=u"dummy", default=u"N/A"),
                   value=u'MyAddress')
        transaction.commit()

        addressblock = self._create_addressblock()
        self.assertEquals('MyAddress', addressblock.Title())

        self._auth()
        self.browser.open(self.contentpage.absolute_url())
        self.assertIn('<h2>MyAddress', self.browser.contents)

    def test_addressblock_change_title(self):
        addressblock = self._create_addressblock()
        addressblock.setTitle('New Title')
        transaction.commit()

        self._auth()
        self.browser.open(self.contentpage.absolute_url())
        self.assertIn('New Title', self.browser.contents)

    def test_default_country(self):
        addressblock = self._create_addressblock()
        self.assertEquals('Switzerland', addressblock.getCountry())

    def test_change_default_country(self):
        registry = getUtility(IRegistry)
        registry.records['ftw.contentpage.addressblock.defaultcountry'] = \
            Record(field.TextLine(title=u"dummy", default=u"N/A"),
                   value=u'Lichtenstein')
        transaction.commit()
        addressblock = self._create_addressblock()
        self.assertEquals('Lichtenstein', addressblock.getCountry())

    def test_exclude_from_nav(self):
        addressblock = self._create_addressblock()
        self.assertTrue(addressblock.getExcludeFromNav())

        self._auth()

    def test_display_opening_houers(self):
        addressblock = self._create_addressblock()
        addressblock.setOpeningHours("Monday to Friday")
        transaction.commit()

        self._auth()
        self.browser.open(self.contentpage.absolute_url())
        self.assertNotIn('<h2>Opening Hours', self.browser.contents)

        addressblock.setShowOpeningHours(True)
        transaction.commit()
        self.browser.open(self.contentpage.absolute_url())
        self.assertIn('<h2>Opening Hours', self.browser.contents)
        self.assertIn('Monday to Friday', self.browser.contents)

    @browsing
    def test_show_map_if_address_is_given(self, browser):
        addressblock = create(Builder('address block')
                              .within(self.contentpage)
                              .having(address="Engehaldenstrasse 53",
                                      zip="3012",
                                      city="Bern"))

        geo = IGeoreferenced(addressblock)
        geo.setGeoInterface('Point', (-105.08, 40.59))

        transaction.commit()

        browser.login().visit(addressblock, view="block_view")
        self.assertEqual(1, len(browser.css('.addressMap')))

        browser.login().visit(addressblock, view="addressblock_detail_view")
        self.assertEqual(1, len(browser.css('.addressMap')))

    @browsing
    def test_hide_map_if_no_address_is_given(self, browser):
        addressblock = create(Builder('address block')
                              .within(self.contentpage))

        browser.login().visit(addressblock, view="block_view")
        self.assertEqual(0, len(browser.css('.addressMap')))

        browser.login().visit(addressblock, view="addressblock_detail_view")
        self.assertEqual(0, len(browser.css('.addressMap')))

    def tearDown(self):
        super(TestAddressBlockCreation, self).tearDown()
        portal = self.layer['portal']
        portal.manage_delObjects(['contentpage'])

        registry = getUtility(IRegistry)
        registry.records['ftw.contentpage.addressblock.defaultcountry'] = \
            Record(field.TextLine(title=u"dummy", default=u"N/A"),
                   value=u'Switzerland')

        transaction.commit()



from ftw.upgrade import UpgradeStep


class AddRegistryEntry(UpgradeStep):
    """Installs the profile which adds a registry entry for default
     table columns
    """
    def __call__(self):
        self.setup_install_profile(
            'profile-ftw.contentpage.upgrades:1003')



from ftw.builder import builder_registry
from ftw.builder.archetypes import ArchetypesBuilder
from ftw.builder.content import ImageBuilder
from ftw.contentpage.portlets import news_portlet
from plone.portlets.interfaces import IPortletAssignmentMapping
from plone.portlets.interfaces import IPortletManager
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component.hooks import getSite
from zope.container.interfaces import INameChooser
from zope.interface import alsoProvides
import transaction


class ContentPageBuilder(ArchetypesBuilder):

    portal_type = 'ContentPage'

    def __init__(self, *args, **kwargs):
        super(ContentPageBuilder, self).__init__(*args, **kwargs)
        self._providing_interfaces = []

    def providing(self, *interfaces):
        self._providing_interfaces.extend(interfaces)
        return self

    def after_create(self, obj):
        if self._providing_interfaces:
            alsoProvides(obj, *self._providing_interfaces)
            obj.reindexObject(idxs=['object_provides'])
        return super(ContentPageBuilder, self).after_create(obj)


builder_registry.register('content page', ContentPageBuilder)


class NewsBuilder(ContentPageBuilder):

    portal_type = 'News'


builder_registry.register('news', NewsBuilder)


class NewsFolderBuilder(ArchetypesBuilder):

    portal_type = 'NewsFolder'


builder_registry.register('news folder', NewsFolderBuilder)


class EventPageBuilder(ContentPageBuilder):

    portal_type = 'EventPage'


builder_registry.register('event page', EventPageBuilder)


class EventFolderBuilder(ArchetypesBuilder):

    portal_type = 'EventFolder'


builder_registry.register('event folder', EventFolderBuilder)


class AddressBlockBuilder(ArchetypesBuilder):

    portal_type = 'AddressBlock'


builder_registry.register('address block', AddressBlockBuilder)


class ListingBlockBuilder(ArchetypesBuilder):

    portal_type = 'ListingBlock'


builder_registry.register('listing block', ListingBlockBuilder)


class TextBlockBuilder(ImageBuilder):

    portal_type = 'TextBlock'

    def attach(self, file_):
        self.arguments['image'] = file_
        return self

builder_registry.register('text block', TextBlockBuilder)


class NewsPortletBuilder(object):

    assignment_class = news_portlet.Assignment

    def __init__(self, session):
        self.session = session
        self.container = getSite()
        self.manager_name = u'plone.leftcolumn'
        self.arguments = {}

    def within(self, container):
        self.container = container
        return self

    def in_manager(self, manager_name):
        self.manager_name = manager_name
        return self

    def having(self, **kwargs):
        self.arguments.update(kwargs)
        return self

    def create(self):
        self.before_create()
        manager, assignments = self.get_manager_and_assignments()
        portlet = self.create_portlet(assignments)
        self.after_create(manager, assignments, portlet)
        return portlet

    def create_portlet(self, assignments):
        portlet = self.assignment_class(**self.arguments)
        name = self.choose_name(assignments, portlet)
        portlet.__name__ = portlet
        assignments[name] = portlet
        return portlet

    def get_manager_and_assignments(self):
        portal = getSite()
        manager = getUtility(IPortletManager,
                             name=self.manager_name,
                             context=portal)
        assignments = getMultiAdapter((self.container, manager),
                                      IPortletAssignmentMapping,
                                      context=portal)
        return manager, assignments

    def choose_name(self, assignments, portlet):
        return INameChooser(assignments).chooseName(
            portlet.__class__.__name__, portlet)

    def before_create(self):
        pass

    def after_create(self, manager, assignments, portlet):
        if self.session.auto_commit:
            transaction.commit()

builder_registry.register('news portlet', NewsPortletBuilder)
